@@ -2,14 +2,11 @@
  *
  * ntriples_parse.c - Raptor N-Triples Parser implementation
  *
- * $Id: ntriples_parse.c,v 1.96 2005/11/03 05:55:07 dajobe Exp $
- *
  * N-Triples
  * http://www.w3.org/TR/rdf-testcases/#ntriples
  *
- * Copyright (C) 2001-2005, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2001-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2001-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -155,7 +152,6 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
   /* raptor_ntriples_parser_context *ntriples_parser=(raptor_ntriples_parser_context*)parser->context; */
   raptor_statement *statement=&parser->statement;
   raptor_uri *subject_uri=NULL;
-  int predicate_ordinal=0;
   raptor_uri *predicate_uri=NULL;
   raptor_uri *object_uri=NULL;
   raptor_uri *datatype_uri=NULL;
@@ -165,47 +161,32 @@ raptor_ntriples_generate_statement(raptor_parser* parser,
     statement->subject=subject;
     statement->subject_type=RAPTOR_IDENTIFIER_TYPE_ANONYMOUS;
   } else {
-    subject_uri=raptor_new_uri_relative_to_base(parser->base_uri, subject);
+    subject_uri=raptor_new_uri(subject);
     statement->subject=subject_uri;
     statement->subject_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
   }
 
  if(object_literal_datatype) {
-   datatype_uri=raptor_new_uri_relative_to_base(parser->base_uri, object_literal_datatype);
+   datatype_uri=raptor_new_uri(object_literal_datatype);
    object_literal_language=NULL;
  }
 
-  /* Predicates in N-Triples are URIs or ordinals */
+  /* Predicates in N-Triples are URIs but check for bad ordinals */
   if(!strncmp((const char*)predicate, "http://www.w3.org/1999/02/22-rdf-syntax-ns#_", 44)) {
-    predicate_ordinal=raptor_check_ordinal(predicate+44);
-    if(predicate_ordinal > 0) {
-      statement->predicate=(void*)&predicate_ordinal;
-      statement->predicate_type=RAPTOR_IDENTIFIER_TYPE_ORDINAL;
-    } else {
+    int predicate_ordinal=raptor_check_ordinal(predicate+44);
+    if(predicate_ordinal <= 0)
       raptor_parser_error(parser, "Illegal ordinal value %d in property '%s'.", predicate_ordinal, predicate);
-      predicate_ordinal=0;
-    }
   }
   
-  if(!predicate_ordinal) {
-    predicate_uri=raptor_new_uri_relative_to_base(parser->base_uri, predicate);
-    statement->predicate_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
-    statement->predicate=predicate_uri;
-  }
+  predicate_uri=raptor_new_uri(predicate);
+  statement->predicate_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
+  statement->predicate=predicate_uri;
   
-  /* FIXME.  Deprecated fixup.
-   * Leave this inplace until depenedent code - rasqal and redland
-   * expect RAPTOR_IDENTIFIER_TYPE_RESOURCE in the predicate position
-   */
-  if(statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_RESOURCE)
-    statement->predicate_type=RAPTOR_IDENTIFIER_TYPE_PREDICATE;
-
   /* Three choices for object from N-Triples */
   statement->object_literal_language=NULL;
   statement->object_literal_datatype=NULL;
   if(object_type == RAPTOR_NTRIPLES_TERM_TYPE_URI_REF) {
-    object_uri=raptor_new_uri_relative_to_base(parser->base_uri, 
-                                               (const unsigned char*)object);
+    object_uri=raptor_new_uri((const unsigned char*)object);
     statement->object=object_uri;
     statement->object_type=RAPTOR_IDENTIFIER_TYPE_RESOURCE;
   } else if(object_type == RAPTOR_NTRIPLES_TERM_TYPE_BLANK_NODE) {
@@ -1073,8 +1054,11 @@ raptor_ntriples_parse_chunk(raptor_parser* rdf_parser,
   
   /* exit now, no more input */
   if(is_end) {
-    if(ntriples_parser->offset != ntriples_parser->line_length)
+    if(ntriples_parser->offset != ntriples_parser->line_length) {
        raptor_parser_error(rdf_parser, "Junk at end of input.\"");
+       return 1;
+    }
+    
     return 0;
   }
     
@@ -1129,12 +1113,17 @@ static void
 raptor_ntriples_parser_register_factory(raptor_parser_factory *factory) 
 {
   factory->context_length     = sizeof(raptor_ntriples_parser_context);
+
+  factory->need_base_uri = 0;
   
   factory->init      = raptor_ntriples_parse_init;
   factory->terminate = raptor_ntriples_parse_terminate;
   factory->start     = raptor_ntriples_parse_start;
   factory->chunk     = raptor_ntriples_parse_chunk;
   factory->recognise_syntax = raptor_ntriples_parse_recognise_syntax;
+
+  raptor_parser_factory_add_uri(factory, 
+                                (const unsigned char*)"http://www.w3.org/TR/rdf-testcases/#ntriples");
 }
 
 
@@ -1142,7 +1131,5 @@ void
 raptor_init_parser_ntriples(void)
 {
   raptor_parser_register_factory("ntriples",  "N-Triples",
-                                 NULL,
-                                 (const unsigned char*)"http://www.w3.org/TR/rdf-testcases/#ntriples",
                                  &raptor_ntriples_parser_register_factory);
 }
