@@ -2,11 +2,8 @@
  *
  * raptor.h - Redland Parser Toolkit for RDF (Raptor) interfaces and definition
  *
- * $Id: raptor.h,v 1.190 2005/11/22 15:53:29 dajobe Exp $
- *
- * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -40,6 +37,13 @@ extern "C" {
 /* Required for va_list in raptor_vsnprintf */
 #include <stdarg.h>
 
+
+/**
+ * RAPTOR_API:
+ *
+ * Macro for wrapping API function call declarations.
+ *
+ */
 #ifdef WIN32
 #  ifdef RAPTOR_STATIC
 #    define RAPTOR_API
@@ -68,6 +72,11 @@ extern "C" {
 #define RAPTOR_DEPRECATED
 #endif
 
+/**
+ * raptor_uri:
+ *
+ * Raptor URI Class.
+ */
 typedef void* raptor_uri;
 
 
@@ -108,22 +117,71 @@ extern const unsigned int raptor_rdf_namespace_uri_len;
 
 RAPTOR_API
 extern const unsigned char * const raptor_xml_literal_datatype_uri_string;
+
+/**
+ * raptor_xml_literal_datatype_uri_string_len:
+ *
+ * Length of #raptor_xml_literal_datatype_uri_string
+ */
 RAPTOR_API
 extern const unsigned int raptor_xml_literal_datatype_uri_string_len;
 
 
 /* Public structure */
+/**
+ * raptor_parser:
+ *
+ * Raptor Parser class
+ */
 typedef struct raptor_parser_s raptor_parser;
+/**
+ * raptor_serializer:
+ *
+ * Raptor Serializer class
+ */
 typedef struct raptor_serializer_s raptor_serializer;
 
+/**
+ * raptor_www:
+ *
+ * Raptor WWW class
+ */
 typedef struct raptor_www_s raptor_www;
+/**
+ * raptor_iostream:
+ *
+ * Raptor I/O Stream class
+ */
 typedef struct raptor_iostream_s raptor_iostream;
-
+/**
+ * raptor_xml_element:
+ *
+ * Raptor XML Element class
+ */
 typedef struct raptor_xml_element_s raptor_xml_element;
+/**
+ * raptor_xml_writer:
+ *
+ * Raptor XML Writer class
+ */
 typedef struct raptor_xml_writer_s raptor_xml_writer;
-
+/**
+ * raptor_qname:
+ *
+ * Raptor XML qname class
+ */
 typedef struct raptor_qname_s raptor_qname;
+/**
+ * raptor_namespace:
+ *
+ * Raptor XML Namespace class
+ */
 typedef struct raptor_namespace_s raptor_namespace;
+/**
+ * raptor_namespace_stack:
+ *
+ * Raptor XML Namespace Stack class
+ */
 typedef struct raptor_namespace_stack_s raptor_namespace_stack;
 
 /**
@@ -140,7 +198,7 @@ typedef raptor_parser raptor_ntriples_parser;
  * @RAPTOR_IDENTIFIER_TYPE_RESOURCE:    Resource URI (e.g. <literal>rdf:about</literal>)
  * @RAPTOR_IDENTIFIER_TYPE_ANONYMOUS:   <literal>_:foo</literal> N-Triples, or generated
  * @RAPTOR_IDENTIFIER_TYPE_PREDICATE:   predicate URI.  WARNING: Will not be generated in in Raptor 1.4.9 or newer.  Instead a #RAPTOR_IDENTIFIER_RESOURCE will be returned.
- * @RAPTOR_IDENTIFIER_TYPE_ORDINAL:     <literal>rdf:li</literal>, <literal>rdf:_</literal><emphasis>n</emphasis>.  May not be generated in Raptor 1.4.9 or newer, instead a #RAPTOR_IDENTIFIER_RESOURCE will be returned.
+ * @RAPTOR_IDENTIFIER_TYPE_ORDINAL:     <literal>rdf:li</literal>, <literal>rdf:_</literal><emphasis>n</emphasis>.  No longer generated in any parser in Raptor 1.4.10+, instead a #RAPTOR_IDENTIFIER_RESOURCE is returned.
  * @RAPTOR_IDENTIFIER_TYPE_LITERAL:     regular literal
  * @RAPTOR_IDENTIFIER_TYPE_XML_LITERAL: <literal>rdf:parseType="Literal"</literal>.  No longer generated by any parser in Raptor 1.4.8+, instead a #RAPTOR_IDENTIFIER_TYPE_LITERAL is returned with a datatype of <literal>rdf:XMLLiteral</literal>.
  * @RAPTOR_IDENTIFIER_TYPE_UNKNOWN:     Internal
@@ -259,6 +317,7 @@ typedef struct {
  *   for each indent level when serializing with auto indent.
  * @RAPTOR_FEATURE_WRITER_XML_VERSION: Integer XML version XML 1.0 (10) or XML 1.1 (11)
  * @RAPTOR_FEATURE_WRITER_XML_DECLARATION: Write XML 1.0 or 1.1 declaration.
+ * @RAPTOR_FEATURE_NO_NET: Deny network requests
  * @RAPTOR_FEATURE_LAST: Internal
  *
  * Raptor parser, serializer or XML writer features.
@@ -281,7 +340,8 @@ typedef enum {
   RAPTOR_FEATURE_WRITER_INDENT_WIDTH,
   RAPTOR_FEATURE_WRITER_XML_VERSION,
   RAPTOR_FEATURE_WRITER_XML_DECLARATION,
-  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_WRITER_XML_DECLARATION
+  RAPTOR_FEATURE_NO_NET,
+  RAPTOR_FEATURE_LAST=RAPTOR_FEATURE_NO_NET
 } raptor_feature;
 
 
@@ -330,7 +390,22 @@ typedef struct {
 } raptor_identifier;
 
 
-/* Returned by statement_handler */
+/**
+ * raptor_statement:
+ * @subject: triple subject data
+ * @subject_type: triple subject type
+ * @predicate: triple predicate data
+ * @predicate_type: triple predicate type
+ * @object: triple object literal string
+ * @object_type: triple object type
+ * @object_literal_datatype: triple object literal datatype URI (or NULL)
+ * @object_literal_language: triple object literal language string (or NULL)
+ *
+ * An RDF triple
+ *
+ * See #raptor_identifier for a description of how the fields may be used.
+ * As returned by a parser statement_handler.
+ */
 typedef struct {
   const void *subject;
   raptor_identifier_type subject_type;
@@ -570,6 +645,17 @@ typedef void (*raptor_www_write_bytes_handler)(raptor_www* www, void *userdata,
  */
 typedef void (*raptor_www_content_type_handler)(raptor_www* www, void *userdata, const char *content_type);
 
+/**
+ * raptor_uri_filter_func:
+ * @user_data: user data
+ * @uri: #raptor_uri URI to check
+ *
+ * Callback function for #raptor_www_set_uri_filter
+ *
+ * Return value: non-0 to filter the URI
+ */
+typedef int (*raptor_uri_filter_func)(void *user_data, raptor_uri* uri);
+
 
 /* Public functions */
 
@@ -614,6 +700,8 @@ RAPTOR_API
 void raptor_set_generate_id_handler(raptor_parser* parser, void *user_data, raptor_generate_id_handler handler);
 RAPTOR_API
 void raptor_set_namespace_handler(raptor_parser* parser, void *user_data, raptor_namespace_handler handler);
+RAPTOR_API
+void raptor_parser_set_uri_filter(raptor_parser* parser, raptor_uri_filter_func filter, void* user_data);
 
 RAPTOR_API
 void raptor_print_statement(const raptor_statement * statement, FILE *stream);
@@ -669,6 +757,8 @@ RAPTOR_API
 const char* raptor_get_label(raptor_parser *rdf_parser);
 RAPTOR_API
 const char* raptor_get_mime_type(raptor_parser *rdf_parser);
+RAPTOR_API
+int raptor_get_need_base_uri(raptor_parser *rdf_parser);
 
 RAPTOR_API
 int raptor_features_enumerate(const raptor_feature feature, const char **name, raptor_uri **uri, const char **label);
@@ -681,7 +771,11 @@ int raptor_get_feature(raptor_parser *parser, raptor_feature feature);
 RAPTOR_API
 const unsigned char* raptor_parser_get_feature_string(raptor_parser *parser, raptor_feature feature);
 RAPTOR_API
+unsigned int raptor_get_feature_count(void);
+RAPTOR_API
 void raptor_set_parser_strict(raptor_parser* rdf_parser, int is_strict);
+RAPTOR_API
+const char* raptor_parser_get_accept_header(raptor_parser* rdf_parser);
 
 /* Get serializer names */
 RAPTOR_API
@@ -706,6 +800,8 @@ int raptor_serialize_start_to_file_handle(raptor_serializer *rdf_serializer, rap
 RAPTOR_API
 int raptor_serialize_set_namespace(raptor_serializer* rdf_serializer, raptor_uri *uri, const unsigned char *prefix);
 RAPTOR_API
+int raptor_serialize_set_namespace_from_namespace(raptor_serializer* rdf_serializer, raptor_namespace *nspace);
+RAPTOR_API
 int raptor_serialize_statement(raptor_serializer* rdf_serializer, const raptor_statement *statement);
 RAPTOR_API
 int raptor_serialize_end(raptor_serializer *rdf_serializer);
@@ -810,9 +906,11 @@ RAPTOR_API
 char *raptor_uri_uri_string_to_filename(const unsigned char *uri_string);
 RAPTOR_API
 char *raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string, unsigned char **fragment_p);
-RAPTOR_API
+RAPTOR_API RAPTOR_DEPRECATED
 int raptor_uri_is_file_uri(const unsigned char* uri_string);
 RAPTOR_API
+int raptor_uri_uri_string_is_file_uri(const unsigned char* uri_string);
+RAPTOR_API
 unsigned char* raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri, raptor_uri *reference_uri, size_t *length_p);
 RAPTOR_API
 unsigned char* raptor_uri_to_relative_uri_string(raptor_uri *base_uri,  raptor_uri *reference_uri);
@@ -824,7 +922,7 @@ RAPTOR_API
 unsigned char* raptor_uri_to_string(raptor_uri *uri);
 
 RAPTOR_API
-void raptor_uri_set_handler(raptor_uri_handler *handler, void *context);
+void raptor_uri_set_handler(const raptor_uri_handler *handler, void *context);
 RAPTOR_API
 void raptor_uri_get_handler(raptor_uri_handler **handler, void **context);
 
@@ -838,7 +936,7 @@ void raptor_uri_get_handler(raptor_uri_handler **handler, void **context);
 #define RAPTOR_RDF_MS_URI raptor_rdf_namespace_uri
 
 /**
- * RAPTOR_RDF_sCHEMA_URI:
+ * RAPTOR_RDF_SCHEMA_URI:
  *
  * RDF Schema Namespace URI (rdfs:).
  *
@@ -847,7 +945,7 @@ void raptor_uri_get_handler(raptor_uri_handler **handler, void **context);
 #define RAPTOR_RDF_SCHEMA_URI raptor_rdf_schema_namespace_uri
 
 /**
- * RAPTOR_XML_sCHEMA_DATATYPES_URI:
+ * RAPTOR_XMLSCHEMA_DATATYPES_URI:
  *
  * XML Schema Datatypes URI (xsd:).
  *
@@ -893,6 +991,8 @@ void raptor_www_set_content_type_handler(raptor_www *www, raptor_www_content_typ
 RAPTOR_API
 void raptor_www_set_error_handler(raptor_www *www, raptor_message_handler error_handler, void *error_data);
 RAPTOR_API
+void raptor_www_set_uri_filter(raptor_www* www, raptor_uri_filter_func filter, void* user_data);
+RAPTOR_API
 int raptor_www_fetch(raptor_www *www, raptor_uri *uri);
 RAPTOR_API
 int raptor_www_fetch_to_string(raptor_www *www, raptor_uri *uri, void **string_p, size_t *length_p, void *(*malloc_handler)(size_t size));
@@ -918,6 +1018,8 @@ RAPTOR_API
 raptor_uri* raptor_qname_string_to_uri(raptor_namespace_stack *nstack,  const unsigned char *name, size_t name_len, raptor_simple_message_handler error_handler, void *error_data);
 RAPTOR_API
 int raptor_iostream_write_qname(raptor_iostream* iostr, raptor_qname *qname);
+RAPTOR_API
+const raptor_namespace* raptor_qname_get_namespace(raptor_qname* name);
 
 /* raptor_namespace_stack - stack of XML namespaces */
 RAPTOR_API
@@ -945,6 +1047,8 @@ RAPTOR_API
 raptor_namespace* raptor_namespaces_find_namespace_by_uri(raptor_namespace_stack *nstack, raptor_uri *ns_uri);
 RAPTOR_API
 int raptor_namespaces_namespace_in_scope(raptor_namespace_stack *nstack, const raptor_namespace *nspace);
+RAPTOR_API
+raptor_qname* raptor_namespaces_qname_from_uri(raptor_namespace_stack *nstack,  raptor_uri *uri, int xml_version);
 
 /* raptor_namespace - XML namespace */
 RAPTOR_API
@@ -958,15 +1062,27 @@ raptor_uri* raptor_namespace_get_uri(const raptor_namespace *ns);
 RAPTOR_API
 const unsigned char* raptor_namespace_get_prefix(const raptor_namespace *ns);
 RAPTOR_API
+const unsigned char* raptor_namespace_get_counted_prefix(const raptor_namespace *ns, size_t *length_p);
+RAPTOR_API
 unsigned char *raptor_namespaces_format(const raptor_namespace *ns, size_t *length_p);
 RAPTOR_API
 int raptor_iostream_write_namespace(raptor_iostream* iostr, raptor_namespace *ns);
 RAPTOR_API
 int raptor_new_namespace_parts_from_string(const unsigned char *string, unsigned char **prefix, unsigned char **uri_string);
 
+/**
+ * raptor_stringbuffer:
+ *
+ * Raptor string buffer class
+ */
 typedef struct raptor_stringbuffer_s raptor_stringbuffer;
 
 /* Sequence class */
+/**
+ * raptor_sequence:
+ *
+ * Raptor sequence class
+ */
 typedef struct raptor_sequence_s raptor_sequence;
 
 /**
@@ -1142,7 +1258,7 @@ typedef struct {
 
 
 RAPTOR_API
-raptor_iostream* raptor_new_iostream_from_handler(void *context, raptor_iostream_handler *handler);
+raptor_iostream* raptor_new_iostream_from_handler(void *context, const raptor_iostream_handler *handler);
 RAPTOR_API
 raptor_iostream* raptor_new_iostream_to_sink(void);
 RAPTOR_API
@@ -1171,6 +1287,8 @@ RAPTOR_API
 int raptor_iostream_format_hexadecimal(raptor_iostream* iostr, unsigned int integer, int width);
 RAPTOR_API
 int raptor_iostream_write_stringbuffer(raptor_iostream* iostr, raptor_stringbuffer *sb);
+RAPTOR_API
+int raptor_iostream_write_uri(raptor_iostream *iostr,  raptor_uri *uri);
 
 /* Parser and Serializer features */
 RAPTOR_API
@@ -1188,9 +1306,15 @@ raptor_qname* raptor_xml_element_get_name(raptor_xml_element *xml_element);
 RAPTOR_API
 void raptor_xml_element_set_attributes(raptor_xml_element* xml_element, raptor_qname **attributes, int count);
 RAPTOR_API
-void raptor_xml_element_declare_namespace(raptor_xml_element* xml_element, raptor_namespace *nspace);
+raptor_qname** raptor_xml_element_get_attributes(raptor_xml_element* xml_element);
+RAPTOR_API
+int raptor_xml_element_get_attributes_count(raptor_xml_element* xml_element);
+RAPTOR_API
+int raptor_xml_element_declare_namespace(raptor_xml_element* xml_element, raptor_namespace *nspace);
 RAPTOR_API
 int raptor_iostream_write_xml_element(raptor_iostream *iostr, raptor_xml_element *element, raptor_namespace_stack *nstack, int is_empty, int is_end, raptor_simple_message_handler error_handler, void *error_data, int depth);
+RAPTOR_API
+int raptor_xml_element_is_empty(raptor_xml_element* xml_element);
 
 /* XML Writer Class (raptor_xml_writer) */
 RAPTOR_API
