@@ -2,11 +2,8 @@
  *
  * raptor_internal.h - Redland Parser Toolkit for RDF (Raptor) internals
  *
- * $Id: raptor_internal.h,v 1.158 2005/11/21 03:52:05 dajobe Exp $
- *
- * Copyright (C) 2002-2004, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2004, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -299,62 +296,8 @@ struct raptor_parser_s {
   /* static statement for use in passing to user code */
   raptor_statement statement;
 
-  /* FEATURE: 
-   * non 0 if scanning for <rdf:RDF> element inside the XML,
-   * else require rdf:RDF as first element. See also feature_assume_is_rdf
-   */
-  int feature_scanning_for_rdf_RDF;
-
-  /* FEATURE:
-   * non 0 to allow non-namespaced resource, ID etc attributes
-   * on RDF namespaced-elements
-   */
-  int feature_allow_non_ns_attributes;
-
-
-  /* FEATURE:
-   * non 0 to handle other rdf:parseType values that are not
-   * literal or resource
-   */
-  int feature_allow_other_parseTypes;
-
-
-  /* FEATURE: 
-   * non 0 if rdf:bagID is allowed.
-   * RDF/XML Syntax (Revised) 2003 forbids this.
-   */
-  int feature_allow_bagID;
-
-  /* FEATURE: 
-   * non 0 if rdf:Collection generates the <idList> rdf:type rdf:List triple.
-   * post-last call WD of RDF/XML Syntax (Revised) 2003 forbids amended
-   * the rules to not generate this.
-   */
-  int feature_allow_rdf_type_rdf_List;
-
-  /* FEATURE: 
-   * non 0 if normalizing xml:lang attribute values to lowercase.
-   * http://www.w3.org/TR/rdf-concepts/#dfn-language-identifier
-   * says this is done when language-tagged literals are in the graph.
-   */
-  int feature_normalize_language;
-
-  /* FEATURE
-   * non 0 if non-NFC literals cause a fatal error rather than
-   * a warning.
-   */
-  int feature_non_nfc_fatal;
-
-  /* FEATURE:
-   * non 0 to give a warning when other rdf:parseType values that are not
-   * Literal, Resource or Collection are seen.
-   */
-  int feature_warn_other_parseTypes;
-
-  /* FEATURE:
-   * non 0 to check rdf:ID values for duplicates
-   */
-  int feature_check_rdf_id;
+  /* Features */
+  int features[RAPTOR_FEATURE_LAST+1];
 
   /* stuff for our user */
   void *user_data;
@@ -379,6 +322,9 @@ struct raptor_parser_s {
   char *default_generate_id_handler_prefix;
   size_t default_generate_id_handler_prefix_length;
 
+  void* uri_filter_user_data;
+  raptor_uri_filter_func uri_filter;
+
   /* parser specific stuff */
   void *context;
 
@@ -391,6 +337,15 @@ struct raptor_parser_s {
 };
 
 
+/** A list of (MIME Type, Q) values */
+struct raptor_type_q_s {
+  const char* mime_type;
+  size_t mime_type_len;
+  int q; /* 0-10 standing for 0.0-1.0 */
+};
+typedef struct raptor_type_q_s raptor_type_q;
+
+
 /** A Parser Factory for a syntax */
 struct raptor_parser_factory_s {
   struct raptor_parser_factory_s* next;
@@ -402,8 +357,10 @@ struct raptor_parser_factory_s {
 
   /* syntax readable label */
   const char* label;
+
   /* syntax MIME type (or NULL) */
-  const char* mime_type;
+  raptor_sequence* mime_types;
+
   /* syntax URI (or NULL) */
   const unsigned char* uri_string;
   
@@ -435,6 +392,11 @@ struct raptor_parser_factory_s {
   /* get the Content-Type value of a URI request */
   void (*content_type_handler)(raptor_parser* rdf_parser, const char* content_type);
 
+  /* get the Accept header of a URI request (OPTIONAL) */
+  const char* (*accept_header)(raptor_parser* rdf_parser);
+
+  /* non-0 if this parser needs a base URI */
+  int need_base_uri;
 };
 
 
@@ -525,6 +487,9 @@ struct raptor_serializer_factory_s {
   
   /* finish the serializer factory */
   void (*finish_factory)(raptor_serializer_factory* factory);
+
+  /* add a namespace using an existing namespace */
+  int (*declare_namespace_from_namespace)(raptor_serializer* serializer, raptor_namespace *nspace);
 };
 
 
@@ -534,12 +499,12 @@ void raptor_serializer_register_factory(const char *name, const char *label, con
 
 /* raptor_general.c */
 
-raptor_parser_factory* raptor_parser_register_factory(const char *name, const char *label, const char *mime_type, const unsigned char *uri_string, void (*factory) (raptor_parser_factory*));
+raptor_parser_factory* raptor_parser_register_factory(const char *name, const char *label, void (*factory) (raptor_parser_factory*));
 void raptor_parser_factory_add_alias(raptor_parser_factory* factory, const char *alias);
+void raptor_parser_factory_add_mime_type(raptor_parser_factory* factory, const char* mime_type, int q);
+void raptor_parser_factory_add_uri(raptor_parser_factory* factory, const unsigned char *uri_string);
 
 unsigned char* raptor_generate_id(raptor_parser *rdf_parser, const int id_for_bag, unsigned char* user_bnodeid);
-const unsigned char* raptor_inscope_xml_language(raptor_parser *rdf_parser);
-raptor_uri* raptor_inscope_base_uri(raptor_parser *rdf_parser);
 #ifdef RAPTOR_DEBUG
 void raptor_stats_print(raptor_parser *rdf_parser, FILE *stream);
 #endif
@@ -548,9 +513,16 @@ raptor_statement* raptor_statement_copy(const raptor_statement *statement);
 void raptor_free_statement(raptor_statement *statement);
 
 /* raptor_parse.c */
+raptor_parser_factory* raptor_get_parser_factory(const char *name);  
 void raptor_delete_parser_factories(void);
+void raptor_free_type_q(raptor_type_q* type_q);
+const char* raptor_parser_get_accept_header_all(void);
+int raptor_parse_uri_no_net_filter(void *user_data, raptor_uri* uri);
 
 /* raptor_general.c */
+void raptor_invoke_message_varargs(const char *type, raptor_message_handler handler, void* user_data, raptor_locator* locator, const char *message, va_list arguments);
+void raptor_invoke_simple_message_varargs(const char *type, raptor_simple_message_handler handler, void* user_data, const char *message, va_list arguments);
+
 extern void raptor_parser_fatal_error(raptor_parser* parser, const char *message, ...);
 extern void raptor_parser_error(raptor_parser* parser, const char *message, ...);
 extern void raptor_parser_simple_error(void* parser, const char *message, ...);
@@ -565,8 +537,6 @@ void raptor_parser_warning_message_handler(void *user_data, raptor_locator* loca
 
 /* raptor_parse.c */
 
-int raptor_parser_exec(raptor_parser* rdf_parser, const char *name);
-
 typedef struct raptor_rdfxml_parser_s raptor_rdfxml_parser;
 
 /* Prototypes for common expat/libxml parsing event-handling functions */
@@ -651,19 +621,23 @@ void raptor_qname_print(FILE *stream, raptor_qname* name);
 
 /* raptor_uri.c */
 void raptor_uri_init(void);
+raptor_uri* raptor_new_uri_from_rdf_ordinal(int ordinal);
 
-void raptor_uri_init_default_handler(raptor_uri_handler *handler);
-
-/* raptor_parse.c */
+/* parsers */
 void raptor_init_parser_rdfxml(void);
 void raptor_init_parser_ntriples(void);
 void raptor_init_parser_turtle(void);
 void raptor_init_parser_n3(void);
 void raptor_init_parser_grddl(void);
 void raptor_init_parser_guess(void);
+void raptor_init_parser_rss(void);
+
+/* raptor_parse.c */
+void raptor_parsers_init(void);
+void raptor_parsers_finish(void);
+
 
 /* raptor_rss.c */
-void raptor_init_parser_rss(void);
 void raptor_init_serializer_rss10(void);
 void raptor_init_serializer_atom(void);
 
@@ -674,11 +648,16 @@ raptor_uri_detail* raptor_new_uri_detail(const unsigned char *uri_string);
 void raptor_free_uri_detail(raptor_uri_detail* uri_detail);
 unsigned char* raptor_uri_detail_to_string(raptor_uri_detail *ud, size_t* len_p);
 
-/* raptor_serializer.c */
+/* serializers */
 void raptor_init_serializer_rdfxml(void);
 void raptor_init_serializer_ntriples(void);
 void raptor_init_serializer_simple(void);
 void raptor_delete_serializer_factories(void);
+
+/* raptor_serializer.c */
+void raptor_serializers_init(void);
+void raptor_serializers_finish(void);
+
 void raptor_serializer_error(raptor_serializer* serializer, const char *message, ...);
 void raptor_serializer_simple_error(void* serializer, const char *message, ...);
 void raptor_serializer_error_varargs(raptor_serializer* serializer, const char *message,  va_list arguments);
@@ -721,7 +700,6 @@ struct  raptor_www_s {
 
 #ifdef RAPTOR_WWW_LIBCURL
   CURL* curl_handle;
-  CURLcode status;
   char error_buffer[CURL_ERROR_SIZE];
   int curl_init_here;
 #endif
@@ -750,6 +728,9 @@ struct  raptor_www_s {
   void *error_data;
   raptor_message_handler error_handler;
 
+  void* uri_filter_user_data;
+  raptor_uri_filter_func uri_filter;
+
   /* can be filled with error location information */
   raptor_locator locator;
 
@@ -772,10 +753,6 @@ void raptor_www_curl_init(raptor_www *www);
 void raptor_www_curl_free(raptor_www *www);
 int raptor_www_curl_fetch(raptor_www *www);
 
-void raptor_www_libwww_init(raptor_www *www);
-void raptor_www_libwww_free(raptor_www *www);
-int raptor_www_libwww_fetch(raptor_www *www);
-
 void raptor_www_libfetch_init(raptor_www *www);
 void raptor_www_libfetch_free(raptor_www *www);
 int raptor_www_libfetch_fetch(raptor_www *www);
@@ -814,19 +791,21 @@ struct raptor_xml_element_s {
   unsigned int content_element_seen;
 
   raptor_sequence *declared_nspaces;
+
+  void* user_data;
 };
 
 
 /* start of an element */
-typedef void (*raptor_sax2_start_element_handler)(void *user_data, const unsigned char *name,  const unsigned char **atts);
+typedef void (*raptor_sax2_start_element_handler)(void *user_data, raptor_xml_element *xml_element);
 /* end of an element */
-typedef void (*raptor_sax2_end_element_handler)(void *user_data, const unsigned char *name);
+typedef void (*raptor_sax2_end_element_handler)(void *user_data, raptor_xml_element* xml_element);
 /* characters */
-typedef void (*raptor_sax2_characters_handler)(void *user_data, const unsigned char *s, int len);
+typedef void (*raptor_sax2_characters_handler)(void *user_data, raptor_xml_element* xml_element, const unsigned char *s, int len);
 /* like <![CDATA[...]> */
-typedef void (*raptor_sax2_cdata_handler)(void *user_data, const unsigned char *s, int len);
+typedef void (*raptor_sax2_cdata_handler)(void *user_data, raptor_xml_element* xml_element, const unsigned char *s, int len);
 /* comment */
-typedef void (*raptor_sax2_comment_handler)(void *user_data, const unsigned char *s);
+typedef void (*raptor_sax2_comment_handler)(void *user_data, raptor_xml_element* xml_element, const unsigned char *s);
 /* unparsed (NDATA) entity */
 typedef void (*raptor_sax2_unparsed_entity_decl_handler)(void *user_data, const unsigned char* entityName, const unsigned char* base, const unsigned char* systemId, const unsigned char* publicId, const unsigned char* notationName);
 /* external entity reference */
@@ -891,10 +870,29 @@ struct raptor_sax2_s {
 
   void *warning_data;
   raptor_message_handler warning_handler;
+
+  /* New XML namespace callback */
+  raptor_namespace_handler  namespace_handler;
+
+  /* FEATURE: 
+   * non 0 if require normalizing xml:lang attribute values to lowercase.
+   */
+  int feature_normalize_language;
+
+  /* FEATURE: 
+   * non 0 if network access is denied
+   */
+  int feature_no_net;
+
+  /* stack of namespaces, most recently added at top */
+  raptor_namespace_stack namespaces;
+
+  /* base URI for resolving relative URIs or xml:base URIs */
+  raptor_uri* base_uri;
 };
 
-void raptor_init_sax2(void);
-void raptor_finish_sax2(void);
+void raptor_sax2_init(void);
+void raptor_sax2_finish(void);
 
 raptor_sax2* raptor_new_sax2(void *user_data, void *error_data, raptor_message_handler error_handler, void *fatal_error_data, raptor_message_handler fatal_error_handler, void *warning_data, raptor_message_handler warning_handler);
 void raptor_free_sax2(raptor_sax2 *sax2);
@@ -906,6 +904,7 @@ void raptor_sax2_set_cdata_handler(raptor_sax2* sax2, raptor_sax2_cdata_handler
 void raptor_sax2_set_comment_handler(raptor_sax2* sax2, raptor_sax2_comment_handler handler);
 void raptor_sax2_set_unparsed_entity_decl_handler(raptor_sax2* sax2, raptor_sax2_unparsed_entity_decl_handler handler);
 void raptor_sax2_set_external_entity_ref_handler(raptor_sax2* sax2, raptor_sax2_external_entity_ref_handler handler);
+void raptor_sax2_set_namespace_handler(raptor_sax2* sax2, raptor_namespace_handler handler);
 void raptor_sax2_set_locator(raptor_sax2* sax2, raptor_locator* locator);
 void raptor_sax2_parse_start(raptor_sax2 *sax2, raptor_uri *base_uri);
 int raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer, size_t len, int is_end);
@@ -919,6 +918,7 @@ void raptor_sax2_dec_depth(raptor_sax2* sax2);
 const unsigned char* raptor_sax2_inscope_xml_language(raptor_sax2* sax2);
 raptor_uri* raptor_sax2_inscope_base_uri(raptor_sax2* sax2);
 void raptor_sax2_update_document_locator(raptor_sax2* sax2, raptor_locator* locator);
+int raptor_sax2_set_feature(raptor_sax2* sax2, raptor_feature feature, int value);
   
 #ifdef RAPTOR_DEBUG
 void raptor_print_xml_element(raptor_xml_element *element, FILE* stream);
