@@ -2,11 +2,8 @@
  *
  * raptor_parse.c - Parsers
  *
- * $Id: raptor_parse.c,v 1.311 2005/11/26 06:43:11 dajobe Exp $
- *
- * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -57,50 +54,83 @@
 #ifndef STANDALONE
 
 /* prototypes for helper functions */
-static raptor_parser_factory* raptor_get_parser_factory(const char *name);
 
 
 /* statics */
 
 /* list of parser factories */
-static raptor_parser_factory* parsers=NULL;
+static raptor_sequence* parsers=NULL;
 
 
+/* helper methods */
 
-/* helper functions */
-
-
-/*
- * raptor_delete_parser_factories - helper function to delete all the registered parser factories
- */
-void
-raptor_delete_parser_factories(void)
+static void
+raptor_free_parser_factory(raptor_parser_factory* factory)
 {
-  raptor_parser_factory *factory, *next;
+  if(factory->finish_factory)
+    factory->finish_factory(factory);
   
-  for(factory=parsers; factory; factory=next) {
-    next=factory->next;
-
-    if(factory->finish_factory)
-      factory->finish_factory(factory);
-
-    RAPTOR_FREE(raptor_parser_factory, (void*)factory->name);
-    RAPTOR_FREE(raptor_parser_factory, (void*)factory->label);
-    if(factory->alias)
-      RAPTOR_FREE(raptor_parser_factory, (void*)factory->alias);
-    if(factory->mime_type)
-      RAPTOR_FREE(raptor_parser_factory, (void*)factory->mime_type);
-    if(factory->uri_string)
-      RAPTOR_FREE(raptor_parser_factory, (void*)factory->uri_string);
-
-    RAPTOR_FREE(raptor_parser_factory, factory);
-  }
-  parsers=NULL;
+  RAPTOR_FREE(raptor_parser_factory, (void*)factory->name);
+  RAPTOR_FREE(raptor_parser_factory, (void*)factory->label);
+  if(factory->alias)
+    RAPTOR_FREE(raptor_parser_factory, (void*)factory->alias);
+  if(factory->mime_types)
+    raptor_free_sequence(factory->mime_types);
+  if(factory->uri_string)
+    RAPTOR_FREE(raptor_parser_factory, (void*)factory->uri_string);
+  
+  RAPTOR_FREE(raptor_parser_factory, factory);
 }
 
 
 /* class methods */
 
+void
+raptor_parsers_init(void)
+{
+  parsers=raptor_new_sequence((raptor_sequence_free_handler *)raptor_free_parser_factory, NULL);
+  
+#ifdef RAPTOR_PARSER_RDFXML
+  raptor_init_parser_rdfxml();
+#endif
+
+#ifdef RAPTOR_PARSER_NTRIPLES
+  raptor_init_parser_ntriples();
+#endif
+
+#ifdef RAPTOR_PARSER_N3
+  raptor_init_parser_n3();
+#endif
+
+#ifdef RAPTOR_PARSER_TURTLE
+  raptor_init_parser_turtle();
+#endif
+
+#ifdef RAPTOR_PARSER_RSS
+  raptor_init_parser_rss();
+#endif
+
+#ifdef RAPTOR_PARSER_GRDDL
+  raptor_init_parser_grddl();
+#endif
+
+#ifdef RAPTOR_PARSER_GUESS
+  raptor_init_parser_guess();
+#endif
+}
+
+
+/*
+ * raptor_finish_parsers - delete all the registered parsers
+ */
+void
+raptor_parsers_finish(void)
+{
+  raptor_free_sequence(parsers);
+  parsers=NULL;
+}
+
+
 /*
  * raptor_parser_register_factory:
  * @name: the short syntax name
@@ -116,32 +146,30 @@ raptor_delete_parser_factories(void)
  **/
 raptor_parser_factory*
 raptor_parser_register_factory(const char *name, const char *label,
-                               const char *mime_type,
-                               const unsigned char *uri_string,
                                void (*factory) (raptor_parser_factory*)) 
 {
   raptor_parser_factory *parser, *h;
-  char *name_copy, *label_copy, *mime_type_copy;
-  unsigned char *uri_string_copy;
+  char *name_copy, *label_copy;
+  int i;
   
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG3("Received registration for syntax %s '%s'\n", name, label);
-  RAPTOR_DEBUG4(raptor_parser_register_factory,
-                "MIME type %s, URI %s\n", 
-                (mime_type ? mime_type : "none"),
-                (uri_string ? uri_string : "none"));
 #endif
   
+  for(i=0;
+      (h=(raptor_parser_factory*)raptor_sequence_get_at(parsers, i));
+      i++) {
+    if(!strcmp(h->name, name)) {
+      RAPTOR_FATAL2("parser %s already registered\n", h->name);
+      return NULL;
+    }
+  }
+  
   parser=(raptor_parser_factory*)RAPTOR_CALLOC(raptor_parser_factory, 1,
                                                sizeof(raptor_parser_factory));
   if(!parser)
     RAPTOR_FATAL1("Out of memory\n");
 
-  for(h = parsers; h; h = h->next ) {
-    if(!strcmp(h->name, name))
-      RAPTOR_FATAL2("parser %s already registered\n", h->name);
-  }
-  
   name_copy=(char*)RAPTOR_CALLOC(cstring, strlen(name)+1, 1);
   if(!name_copy) {
     RAPTOR_FREE(raptor_parser, parser);
@@ -158,35 +186,16 @@ raptor_parser_register_factory(const char *name, const char *label,
   strcpy(label_copy, label);
   parser->label=label_copy;
 
-  if(mime_type) {
-    mime_type_copy=(char*)RAPTOR_CALLOC(cstring, strlen(mime_type)+1, 1);
-    if(!mime_type_copy) {
-      RAPTOR_FREE(raptor_parser, parser);
-      RAPTOR_FATAL1("Out of memory\n");
-    }
-    strcpy(mime_type_copy, mime_type);
-    parser->mime_type=mime_type_copy;
-  }
-
-  if(uri_string) {
-    uri_string_copy=(unsigned char*)RAPTOR_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
-    if(!uri_string_copy) {
-    RAPTOR_FREE(raptor_parser, parser);
-    RAPTOR_FATAL1("Out of memory\n");
-    }
-    strcpy((char*)uri_string_copy, (const char*)uri_string);
-    parser->uri_string=uri_string_copy;
-  }
-        
+  parser->mime_types=raptor_new_sequence((raptor_sequence_free_handler*)raptor_free_type_q, NULL);
+  
   /* Call the parser registration function on the new object */
   (*factory)(parser);
   
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG3("%s has context size %d\n", name, parser->context_length);
 #endif
-  
-  parser->next = parsers;
-  parsers = parser;
+
+  raptor_sequence_push(parsers, parser);
 
   return parser;
 }
@@ -198,10 +207,15 @@ raptor_parser_factory_add_alias(raptor_parser_factory* factory,
 {
   raptor_parser_factory *p;
   char *alias_copy;
-
-  for(p = parsers; p; p = p->next ) {
-    if(!strcmp(p->name, alias))
+  int i;
+  
+  for(i=0;
+      (p=(raptor_parser_factory*)raptor_sequence_get_at(parsers, i));
+      i++) {
+    if(!strcmp(p->name, alias)) {
       RAPTOR_FATAL2("parser %s already registered\n", p->name);
+      return;
+    }
   }
   
   alias_copy=(char*)RAPTOR_CALLOC(cstring, strlen(alias)+1, 1);
@@ -213,6 +227,80 @@ raptor_parser_factory_add_alias(raptor_parser_factory* factory,
 }
 
 
+void
+raptor_free_type_q(raptor_type_q* type_q)
+{
+  RAPTOR_FREE(cstring, (void*)type_q->mime_type);
+  RAPTOR_FREE(raptor_type_q, (void*)type_q);
+}
+
+
+/**
+ * raptor_parser_factory_add_mime_type:
+ * @factory: Raptor parser factory
+ * @mime_type: MIME Type string
+ * @q: Accept 'Q' value 0 to 10 inclusive representing 0.0 to 1.0
+ * 
+ * Register a MIME type as handled by a factory.
+ *
+ * The last added MIME type is the default or main one reported.
+ *
+ **/
+void
+raptor_parser_factory_add_mime_type(raptor_parser_factory* factory,
+                                    const char* mime_type, int q)
+{
+  raptor_type_q* type_q;
+  char* mime_type_copy;
+  size_t len;
+  
+  type_q=(raptor_type_q*)RAPTOR_CALLOC(raptor_type_q, sizeof(raptor_type_q), 1);
+  len=strlen(mime_type);
+  mime_type_copy=(char*)RAPTOR_CALLOC(cstring, len+1, 1);
+  if(!mime_type_copy) {
+    RAPTOR_FATAL1("Out of memory\n");
+  }
+  strcpy(mime_type_copy, mime_type);
+
+  type_q->mime_type=mime_type_copy;
+  type_q->mime_type_len=len;
+
+  if(q<0)
+    q=0;
+  if(q>10)
+    q=10;
+  type_q->q=q;
+
+  raptor_sequence_push(factory->mime_types, type_q);
+}
+
+
+/**
+ * raptor_parser_factory_add_uri:
+ * @factory: Raptor parser factory
+ * @uri_string: URI string
+ * 
+ * Register an identifying URI as handled by a factory.
+ *
+ **/
+void
+raptor_parser_factory_add_uri(raptor_parser_factory* factory,
+                              const unsigned char *uri_string)
+{
+  unsigned char *uri_string_copy;
+
+  if(!uri_string)
+    return;
+  
+  uri_string_copy=(unsigned char*)RAPTOR_CALLOC(cstring, strlen((const char*)uri_string)+1, 1);
+  if(!uri_string_copy)
+    RAPTOR_FATAL1("Out of memory\n");
+
+  strcpy((char*)uri_string_copy, (const char*)uri_string);
+  factory->uri_string=uri_string_copy;
+}
+
+
 /**
  * raptor_get_parser_factory:
  * @name: the factory name or NULL for the default factory
@@ -221,20 +309,24 @@ raptor_parser_factory_add_alias(raptor_parser_factory* factory,
  * 
  * Return value: the factory object or NULL if there is no such factory
  **/
-static raptor_parser_factory*
-raptor_get_parser_factory (const char *name) 
+raptor_parser_factory*
+raptor_get_parser_factory(const char *name) 
 {
   raptor_parser_factory *factory;
 
   /* return 1st parser if no particular one wanted - why? */
   if(!name) {
-    factory=parsers;
+    factory=(raptor_parser_factory *)raptor_sequence_get_at(parsers, 0);
     if(!factory) {
       RAPTOR_DEBUG1("No (default) parsers registered\n");
       return NULL;
     }
   } else {
-    for(factory=parsers; factory; factory=factory->next) {
+    int i;
+    
+    for(i=0;
+        (factory=(raptor_parser_factory*)raptor_sequence_get_at(parsers, i));
+        i++) {
       if(!strcmp(factory->name, name) ||
          (factory->alias && !strcmp(factory->alias, name)))
         break;
@@ -268,27 +360,34 @@ raptor_syntaxes_enumerate(const unsigned int counter,
                           const char **mime_type,
                           const unsigned char **uri_string)
 {
-  unsigned int i;
-  raptor_parser_factory *factory=parsers;
+  raptor_parser_factory *factory;
+
+  if(counter < 0)
+    return 1;
+
+  factory=(raptor_parser_factory*)raptor_sequence_get_at(parsers,
+                                                         counter);
 
   if(!factory)
     return 1;
 
-  for(i=0; factory && i<=counter ; i++, factory=factory->next) {
-    if(i == counter) {
-      if(name)
-        *name=factory->name;
-      if(label)
-        *label=factory->label;
-      if(mime_type)
-        *mime_type=factory->mime_type;
-      if(uri_string)
-        *uri_string=factory->uri_string;
-      return 0;
+  if(name)
+    *name=factory->name;
+  if(label)
+    *label=factory->label;
+  if(mime_type) {
+    const char *mime_type_t=NULL;
+    if(factory->mime_types) {
+      raptor_type_q* tq;
+      tq=(raptor_type_q*)raptor_sequence_get_at(factory->mime_types, 0);
+      if(tq)
+        mime_type_t=tq->mime_type;
     }
+    *mime_type=mime_type_t;
   }
-        
-  return 1;
+  if(uri_string)
+    *uri_string=factory->uri_string;
+  return 0;
 }
 
 
@@ -399,70 +498,31 @@ raptor_new_parser_for_content(raptor_uri *uri, const char *mime_type,
 
 
 
-/*
- * raptor_parser_exec:
- * @parser: old parser to change
- * @name: the parser name
- *
- * Overwriting Constructor - turn one parser into another type - internal
- *
- * Return value: non-0 on failure
- */
-int
-raptor_parser_exec(raptor_parser* rdf_parser, const char *name)
-{
-  raptor_parser_factory* factory;
-
-  factory=raptor_get_parser_factory(name);
-  if(!factory)
-    return 1;
-
-  /* Free parser-implementation specific bits */
-  if(rdf_parser->factory)
-    rdf_parser->factory->terminate(rdf_parser);
-
-  if(rdf_parser->context)
-    RAPTOR_FREE(raptor_parser_context, rdf_parser->context);
-
-  rdf_parser->context=(char*)RAPTOR_CALLOC(raptor_parser_context, 1,
-                                           factory->context_length);
-
-  rdf_parser->factory=factory;
-  
-  if(!rdf_parser->context) {
-    raptor_free_parser(rdf_parser);
-    return 1;
-  }
-
-  if(factory->init(rdf_parser, name)) {
-    raptor_free_parser(rdf_parser);
-    return 1;
-  }
-  
-  return 0;
-}
-
-
 /**
  * raptor_start_parse:
  * @rdf_parser: RDF parser
- * @uri: base URI or NULL if no base URI is required
+ * @uri: base URI or may be NULL if no base URI is required
  *
  * Start a parse of content with base URI.
  * 
- * Only the N-Triples parser has an optional base URI.
+ * Parsers that need a base URI can be tested with raptor_get_need_base_uri().
  * 
- * Return value: non-0 on failure.
+ * Return value: non-0 on failure, <0 if a required base URI was missing
  **/
 int
 raptor_start_parse(raptor_parser *rdf_parser, raptor_uri *uri) 
 {
+  if(rdf_parser->factory->need_base_uri && !uri) {
+    raptor_parser_error(rdf_parser, "Missing base URI for %s parser.",
+                        rdf_parser->factory->name);
+    return -1;
+  }
+
   if(uri)
     uri=raptor_uri_copy(uri);
   
   if(rdf_parser->base_uri)
     raptor_free_uri(rdf_parser->base_uri);
-
   rdf_parser->base_uri=uri;
 
   rdf_parser->locator.uri    = uri;
@@ -665,6 +725,20 @@ raptor_parse_uri_content_type_handler(raptor_www* www, void* userdata,
 }
 
 
+int
+raptor_parse_uri_no_net_filter(void *user_data, raptor_uri* uri)
+{
+  unsigned char* uri_string=raptor_uri_as_string(uri);
+  
+  if(raptor_uri_uri_string_is_file_uri(uri_string))
+    return 0;
+
+  raptor_parser_error((raptor_parser*)user_data, 
+                      "Network fetch of URI '%s' denied", uri);
+  return 1;
+}
+
+
 /**
  * raptor_parse_uri:
  * @rdf_parser: parser
@@ -709,7 +783,6 @@ raptor_parse_uri_with_connection(raptor_parser* rdf_parser, raptor_uri *uri,
                                  raptor_uri *base_uri, void *connection)
 {
   raptor_www *www;
-  const char* mime_type;
   
   if(!base_uri)
     base_uri=uri;
@@ -719,18 +792,25 @@ raptor_parse_uri_with_connection(raptor_parser* rdf_parser, raptor_uri *uri,
     if(!www)
       return 1;
   } else {
+    const char *accept_h;
+
     www=raptor_www_new();
     if(!www)
       return 1;
-    if((mime_type=raptor_get_mime_type(rdf_parser))) {
-      char *accept_h=(char*)RAPTOR_MALLOC(cstring, strlen(mime_type)+11);
-      strcpy(accept_h, mime_type);
-      strcat(accept_h, ",*/*;q=0.1"); /* strlen()=4 chars */
+
+    accept_h=raptor_parser_get_accept_header(rdf_parser);
+    if(accept_h) {
       raptor_www_set_http_accept(www, accept_h);
       RAPTOR_FREE(cstring, accept_h);
     }
   }
   
+  if(rdf_parser->uri_filter)
+    raptor_www_set_uri_filter(www, rdf_parser->uri_filter,
+                              rdf_parser->uri_filter_user_data);
+  else if(rdf_parser->features[RAPTOR_FEATURE_NO_NET])
+    raptor_www_set_uri_filter(www, raptor_parse_uri_no_net_filter, rdf_parser);
+  
   raptor_www_set_error_handler(www, rdf_parser->error_handler, 
                                rdf_parser->error_user_data);
   raptor_www_set_write_bytes_handler(www, raptor_parse_uri_write_bytes, 
@@ -855,46 +935,42 @@ raptor_parser_error(raptor_parser* parser, const char *message, ...)
  * raptor_parser_error 
  */
 void
-raptor_parser_simple_error(void* parser, const char *message, ...)
+raptor_parser_simple_error(void* user_data, const char *message, ...)
 {
+  raptor_parser* parser=(raptor_parser*)user_data;
   va_list arguments;
 
   va_start(arguments, message);
 
-  raptor_parser_error_varargs((raptor_parser*)parser, message, arguments);
+  if(parser)
+    raptor_invoke_message_varargs("error",
+                                  parser->error_handler,
+                                  parser->error_user_data,
+                                  &parser->locator,
+                                  message, arguments);
   
   va_end(arguments);
 }
 
 
-/*
- * raptor_parser_error_varargs - Error from a parser - Internal
- */
-void
-raptor_parser_error_varargs(raptor_parser* parser, const char *message, 
+/**
+ * raptor_parser_error_varargs:
+ * @parser: parser
+ * @message: error format message
+ * @arguments: varargs for message
+ *
+ * Error from a parser - Internal.
+ */  
+void  
+raptor_parser_error_varargs(raptor_parser* parser, const char *message,  
                             va_list arguments)
 {
-  if(parser && parser->error_handler) {
-    char *buffer=raptor_vsnprintf(message, arguments);
-    size_t length;
-    if(!buffer) {
-      fprintf(stderr, "raptor_parser_error_varargs: Out of memory\n");
-      return;
-    }
-    length=strlen(buffer);
-    if(buffer[length-1]=='\n')
-      buffer[length-1]='\0';
-    parser->error_handler(parser->error_user_data, 
-                          &parser->locator, buffer);
-    RAPTOR_FREE(cstring, buffer);
-    return;
-  }
-
   if(parser)
-    raptor_print_locator(stderr, &parser->locator);
-  fprintf(stderr, " raptor error - ");
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
+    raptor_invoke_message_varargs("error",
+                                  parser->error_handler,
+                                  parser->error_user_data,
+                                  &parser->locator,
+                                  message, arguments);
 }
 
 
@@ -931,43 +1007,16 @@ raptor_parser_warning(raptor_parser* parser, const char *message, ...)
 
   va_start(arguments, message);
 
-  raptor_parser_warning_varargs(parser, message, arguments);
-
+  if(parser)
+    raptor_invoke_message_varargs("warning",
+                                  parser->warning_handler,
+                                  parser->warning_user_data, &parser->locator,
+                                  message, arguments);
+  
   va_end(arguments);
 }
 
 
-/*
- * raptor_parser_warning - Warning from a parser - Internal
- */
-void
-raptor_parser_warning_varargs(raptor_parser* parser, const char *message, 
-                              va_list arguments)
-{
-
-  if(parser->warning_handler) {
-    char *buffer=raptor_vsnprintf(message, arguments);
-    size_t length;
-    if(!buffer) {
-      fprintf(stderr, "raptor_parser_warning_varargs: Out of memory\n");
-      return;
-    }
-    length=strlen(buffer);
-    if(buffer[length-1]=='\n')
-      buffer[length-1]='\0';
-    parser->warning_handler(parser->warning_user_data,
-                            &parser->locator, buffer);
-    RAPTOR_FREE(cstring, buffer);
-    return;
-  }
-
-  raptor_print_locator(stderr, &parser->locator);
-  fprintf(stderr, " raptor warning - ");
-  vfprintf(stderr, message, arguments);
-  fputc('\n', stderr);
-}
-
-
 /*
  * raptor_parser_warning_message_handler - Warning from a parser - Internal
  */
@@ -1129,6 +1178,24 @@ raptor_set_namespace_handler(raptor_parser* parser,
 }
 
 
+/**
+ * raptor_parser_set_uri_filter:
+ * @parser: parser object
+ * @filter: URI filter function
+ * @user_data: User data to pass to filter function
+ * 
+ * Set URI filter function for WWW retrieval.
+ **/
+void
+raptor_parser_set_uri_filter(raptor_parser* parser, 
+                             raptor_uri_filter_func filter,
+                             void *user_data)
+{
+  parser->uri_filter=filter;
+  parser->uri_filter_user_data=user_data;
+}
+
+
 /**
  * raptor_features_enumerate:
  * @feature: feature enumeration (0+)
@@ -1172,43 +1239,21 @@ raptor_set_feature(raptor_parser *parser, raptor_feature feature, int value)
   
   switch(feature) {
     case RAPTOR_FEATURE_SCANNING:
-      parser->feature_scanning_for_rdf_RDF=value;
+    case RAPTOR_FEATURE_ALLOW_NON_NS_ATTRIBUTES:
+    case RAPTOR_FEATURE_ALLOW_OTHER_PARSETYPES:
+    case RAPTOR_FEATURE_ALLOW_BAGID:
+    case RAPTOR_FEATURE_ALLOW_RDF_TYPE_RDF_LIST:
+    case RAPTOR_FEATURE_NORMALIZE_LANGUAGE:
+    case RAPTOR_FEATURE_NON_NFC_FATAL:
+    case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
+    case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_NO_NET:
+      parser->features[(int)feature]=value;
       break;
 
     case RAPTOR_FEATURE_ASSUME_IS_RDF:
       break;
 
-    case RAPTOR_FEATURE_ALLOW_NON_NS_ATTRIBUTES:
-      parser->feature_allow_non_ns_attributes=value;
-      break;
-
-    case RAPTOR_FEATURE_ALLOW_OTHER_PARSETYPES:
-      parser->feature_allow_other_parseTypes=value;
-      break;
-
-    case RAPTOR_FEATURE_ALLOW_BAGID:
-      parser->feature_allow_bagID=value;
-      break;
-
-    case RAPTOR_FEATURE_ALLOW_RDF_TYPE_RDF_LIST:
-      parser->feature_allow_rdf_type_rdf_List=value;
-      break;
-
-    case RAPTOR_FEATURE_NORMALIZE_LANGUAGE:
-      parser->feature_normalize_language=value;
-      break;
-
-    case RAPTOR_FEATURE_NON_NFC_FATAL:
-      parser->feature_non_nfc_fatal=value;
-      break;
-
-    case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
-      parser->feature_warn_other_parseTypes=value;
-      break;
-
-    case RAPTOR_FEATURE_CHECK_RDF_ID:
-      parser->feature_check_rdf_id=value;
-      break;
 
     case RAPTOR_FEATURE_RELATIVE_URIS:
     case RAPTOR_FEATURE_START_URI:
@@ -1272,45 +1317,22 @@ raptor_get_feature(raptor_parser *parser, raptor_feature feature)
   
   switch(feature) {
     case RAPTOR_FEATURE_SCANNING:
-      result=(parser->feature_scanning_for_rdf_RDF != 0);
+    case RAPTOR_FEATURE_ALLOW_NON_NS_ATTRIBUTES:
+    case RAPTOR_FEATURE_ALLOW_OTHER_PARSETYPES:
+    case RAPTOR_FEATURE_ALLOW_BAGID:
+    case RAPTOR_FEATURE_ALLOW_RDF_TYPE_RDF_LIST:
+    case RAPTOR_FEATURE_NORMALIZE_LANGUAGE:
+    case RAPTOR_FEATURE_NON_NFC_FATAL:
+    case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
+    case RAPTOR_FEATURE_CHECK_RDF_ID:
+    case RAPTOR_FEATURE_NO_NET:
+      result=(parser->features[(int)feature] != 0);
       break;
 
     case RAPTOR_FEATURE_ASSUME_IS_RDF:
       result=0;
       break;
 
-    case RAPTOR_FEATURE_ALLOW_NON_NS_ATTRIBUTES:
-      result=(parser->feature_allow_non_ns_attributes != 0);
-      break;
-
-    case RAPTOR_FEATURE_ALLOW_OTHER_PARSETYPES:
-      result=(parser->feature_allow_other_parseTypes != 0);
-      break;
-
-    case RAPTOR_FEATURE_ALLOW_BAGID:
-      result=(parser->feature_allow_bagID != 0);
-      break;
-
-    case RAPTOR_FEATURE_ALLOW_RDF_TYPE_RDF_LIST:
-      result=(parser->feature_allow_rdf_type_rdf_List != 0);
-      break;
-
-    case RAPTOR_FEATURE_NORMALIZE_LANGUAGE:
-      result=(parser->feature_normalize_language != 0);
-      break;
-
-    case RAPTOR_FEATURE_NON_NFC_FATAL:
-      result=(parser->feature_non_nfc_fatal != 0);
-      break;
-      
-    case RAPTOR_FEATURE_WARN_OTHER_PARSETYPES:
-      result=(parser->feature_warn_other_parseTypes != 0);
-      break;
-
-    case RAPTOR_FEATURE_CHECK_RDF_ID:
-      result=(parser->feature_check_rdf_id != 0);
-      break;
-
     /* serializing features */
     case RAPTOR_FEATURE_RELATIVE_URIS:
     case RAPTOR_FEATURE_START_URI:
@@ -1368,16 +1390,16 @@ raptor_set_parser_strict(raptor_parser* rdf_parser, int is_strict)
   is_strict=(is_strict) ? 1 : 0;
 
   /* Initialise default parser mode */
-  rdf_parser->feature_scanning_for_rdf_RDF=0;
+  rdf_parser->features[RAPTOR_FEATURE_SCANNING]=0;
 
-  rdf_parser->feature_allow_non_ns_attributes=!is_strict;
-  rdf_parser->feature_allow_other_parseTypes=!is_strict;
-  rdf_parser->feature_allow_bagID=!is_strict;
-  rdf_parser->feature_allow_rdf_type_rdf_List=0;
-  rdf_parser->feature_normalize_language=1;
-  rdf_parser->feature_non_nfc_fatal=is_strict;
-  rdf_parser->feature_warn_other_parseTypes=!is_strict;
-  rdf_parser->feature_check_rdf_id=1;
+  rdf_parser->features[RAPTOR_FEATURE_ALLOW_NON_NS_ATTRIBUTES]=!is_strict;
+  rdf_parser->features[RAPTOR_FEATURE_ALLOW_OTHER_PARSETYPES]=!is_strict;
+  rdf_parser->features[RAPTOR_FEATURE_ALLOW_BAGID]=!is_strict;
+  rdf_parser->features[RAPTOR_FEATURE_ALLOW_RDF_TYPE_RDF_LIST]=0;
+  rdf_parser->features[RAPTOR_FEATURE_NORMALIZE_LANGUAGE]=1;
+  rdf_parser->features[RAPTOR_FEATURE_NON_NFC_FATAL]=is_strict;
+  rdf_parser->features[RAPTOR_FEATURE_WARN_OTHER_PARSETYPES]=!is_strict;
+  rdf_parser->features[RAPTOR_FEATURE_CHECK_RDF_ID]=1;
 }
 
 
@@ -1471,7 +1493,30 @@ raptor_get_label(raptor_parser *rdf_parser)
 const char*
 raptor_get_mime_type(raptor_parser *rdf_parser) 
 {
-  return rdf_parser->factory->mime_type;
+  const char *mime_type=NULL;
+  if(rdf_parser->factory->mime_types) {
+    raptor_type_q* tq;
+    tq=(raptor_type_q*)raptor_sequence_get_at(rdf_parser->factory->mime_types, 0);
+    if(tq)
+      mime_type=tq->mime_type;
+  }
+  
+  return mime_type;
+}
+
+
+/**
+ * raptor_get_need_base_uri:
+ * @rdf_parser: #raptor_parser parser object
+ *
+ * Get a boolean whether this parser needs a base URI to start parsing.
+ *
+ * Return value: non-0 if this parser needs a base URI
+ **/
+int
+raptor_get_need_base_uri(raptor_parser *rdf_parser)
+{
+  return rdf_parser->factory->need_base_uri;
 }
 
 
@@ -1619,7 +1664,7 @@ raptor_guess_parser_name(raptor_uri *uri, const char *mime_type,
                          const unsigned char *identifier)
 {
   unsigned int i;
-  raptor_parser_factory *factory=parsers;
+  raptor_parser_factory *factory;
   unsigned char *suffix=NULL;
 /* FIXME - up to 10 parsers :) */
 #define MAX_PARSERS 10
@@ -1641,12 +1686,26 @@ raptor_guess_parser_name(raptor_uri *uri, const char *mime_type,
     }
   }
 
-  for(i=0; factory; i++, factory=factory->next) {
+  for(i=0;
+      (factory=(raptor_parser_factory*)raptor_sequence_get_at(parsers, i));
+      i++) {
     int score= -1;
+    raptor_type_q* type_q=NULL;
     
-    if(mime_type && factory->mime_type &&
-       !strcmp(mime_type, factory->mime_type))
-      /* got an exact match mime type - return result */
+    if(mime_type && factory->mime_types) {
+      int j;
+      type_q=NULL;
+      for(j=0; 
+          (type_q=(raptor_type_q*)raptor_sequence_get_at(factory->mime_types, j)); 
+          j++) {
+        if(!strcmp(mime_type, type_q->mime_type))
+          break;
+      }
+      if(type_q)
+        break;
+    }
+    /* got an exact match mime type - return result */
+    if(type_q)
       break;
     
     if(uri && factory->uri_string &&
@@ -1696,6 +1755,8 @@ void
 raptor_parser_copy_user_state(raptor_parser *to_parser, 
                               raptor_parser *from_parser)
 {
+  int i;
+  
   to_parser->user_data= from_parser->user_data;
   to_parser->fatal_error_user_data= from_parser->fatal_error_user_data;
   to_parser->error_user_data= from_parser->error_user_data;
@@ -1711,6 +1772,12 @@ raptor_parser_copy_user_state(raptor_parser *to_parser,
   to_parser->default_generate_id_handler_prefix_length= from_parser->default_generate_id_handler_prefix_length;
   to_parser->namespace_handler= from_parser->namespace_handler;
   to_parser->namespace_handler_user_data= from_parser->namespace_handler_user_data;
+  to_parser->uri_filter= from_parser->uri_filter;
+  to_parser->uri_filter_user_data= from_parser->uri_filter_user_data;
+
+  /* copy features */
+  for(i=0; i< RAPTOR_FEATURE_LAST; i++)
+    to_parser->features[i]= from_parser->features[i];
 
 }
 
@@ -1734,6 +1801,143 @@ raptor_parser_start_namespace(raptor_parser* rdf_parser,
 }
 
 
+/**
+ * raptor_parser_get_accept_header:
+ * @rdf_parser: parser
+ * 
+ * Get an HTTP Accept value for the parser.
+ *
+ * The returned string must be freed by the caller such as with
+ * raptor_free_memory().
+ *
+ * Return value: a new Accept: header string or NULL on failure
+ **/
+const char*
+raptor_parser_get_accept_header(raptor_parser* rdf_parser)
+{
+  raptor_parser_factory *factory=rdf_parser->factory;
+  char *accept_header=NULL;
+  size_t len;
+  char *p;
+  int i;
+  raptor_type_q* type_q;
+  
+  if(factory->accept_header)
+    return factory->accept_header(rdf_parser);
+
+  if(!factory->mime_types)
+    return NULL;
+
+  len=0;
+  for(i=0; 
+      (type_q=(raptor_type_q*)raptor_sequence_get_at(factory->mime_types, i));
+      i++) {
+    if(type_q->mime_type) {
+      len+= type_q->mime_type_len + 2; /* ", " */
+      if(type_q->q < 10)
+        len+= 6; /* ";q=X.Y" */
+    }
+  }
+  
+  /* 9 = "\*\/\*;q=0.1" */
+  accept_header=(char*)RAPTOR_MALLOC(cstring, len + 9 + 1);
+  if(!accept_header)
+    return NULL;
+
+  p=accept_header;
+  for(i=0; 
+      (type_q=(raptor_type_q*)raptor_sequence_get_at(factory->mime_types, i));
+      i++) {
+    if(type_q->mime_type) {
+      strncpy(p, type_q->mime_type, type_q->mime_type_len);
+      p+= type_q->mime_type_len;
+      if(type_q->q < 10) {
+        *p++ = ';';
+        *p++ = 'q';
+        *p++ = '=';
+        *p++ = '0';
+        *p++ = '.';
+        *p++ = '0' + (type_q->q);
+      }
+    }
+    
+    *p++ = ',';
+    *p++ = ' ';
+  }
+
+  strncpy(p, "*/*;q=0.1", 10);
+
+  return accept_header;
+}
+
+
+const char*
+raptor_parser_get_accept_header_all(void)
+{
+  raptor_parser_factory *factory;
+  char *accept_header=NULL;
+  size_t len;
+  char *p;
+  int i;
+  
+  len=0;
+  for(i=0;
+      (factory=(raptor_parser_factory*)raptor_sequence_get_at(parsers, i));
+      i++) {
+    raptor_type_q* type_q;
+    int j;
+    
+    for(j=0;
+        (type_q=(raptor_type_q*)raptor_sequence_get_at(factory->mime_types, j));
+        j++) {
+      if(type_q->mime_type) {
+        len+= type_q->mime_type_len + 2; /* ", " */
+        if(type_q->q < 10)
+          len+= 6; /* ";q=X.Y" */
+      }
+    }
+  }
+  
+  /* 9 = "\*\/\*;q=0.1" */
+  accept_header=(char*)RAPTOR_MALLOC(cstring, len + 9 + 1);
+  if(!accept_header)
+    return NULL;
+
+  p=accept_header;
+  for(i=0;
+      (factory=(raptor_parser_factory*)raptor_sequence_get_at(parsers, i));
+      i++) {
+    raptor_type_q* type_q;
+    int j;
+    
+    for(j=0; 
+        (type_q=(raptor_type_q*)raptor_sequence_get_at(factory->mime_types, j));
+        j++) {
+      if(type_q->mime_type) {
+        strncpy(p, type_q->mime_type, type_q->mime_type_len);
+        p+= type_q->mime_type_len;
+        if(type_q->q < 10) {
+          *p++ = ';';
+          *p++ = 'q';
+          *p++ = '=';
+          *p++ = '0';
+          *p++ = '.';
+          *p++ = '0' + (type_q->q);
+        }
+      }
+      
+      *p++ = ',';
+      *p++ = ' ';
+    }
+    
+  }
+  
+  strncpy(p, "*/*;q=0.1", 10);
+  
+  return accept_header;
+}
+
+
 /* end not STANDALONE */
 #endif
 
@@ -1749,6 +1953,7 @@ main(int argc, char *argv[])
 {
   const char *program=raptor_basename(argv[0]);
   int i;
+  const char *s;
 
   raptor_init();
   
@@ -1756,7 +1961,7 @@ main(int argc, char *argv[])
   fprintf(stderr, "%s: Known features:\n", program);
 #endif
 
-  for(i=0; 1; i++) {
+  for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
     const char *feature_name;
     const char *feature_label;
     raptor_uri *feature_uri;
@@ -1764,7 +1969,7 @@ main(int argc, char *argv[])
     
     if(raptor_features_enumerate((raptor_feature)i,
                                  &feature_name, &feature_uri, &feature_label))
-      break;
+      continue;
 
 #ifdef RAPTOR_DEBUG
     fprintf(stderr, " %2d %-20s %s\n", i, feature_name, feature_label);
@@ -1777,6 +1982,10 @@ main(int argc, char *argv[])
     raptor_free_uri(feature_uri);
   }
 
+  s=raptor_parser_get_accept_header_all();
+  fprintf(stderr, "Default HTTP accept header: '%s'\n", s);
+  RAPTOR_FREE(cstring, s);
+
   raptor_finish();
   
   return 0;
