@@ -53,7 +53,7 @@
 
 
 void
-raptor_init_sax2(void)
+raptor_sax2_init(void)
 {
 #ifdef RAPTOR_XML_LIBXML
   xmlInitParser();
@@ -62,7 +62,7 @@ raptor_init_sax2(void)
 
 
 void
-raptor_finish_sax2(void)
+raptor_sax2_finish(void)
 {
 #ifdef RAPTOR_XML_LIBXML
   xmlCleanupParser();
@@ -242,6 +242,14 @@ raptor_xml_element_push(raptor_sax2 *sax2, raptor_xml_element* element)
 }
 
 
+/**
+ * raptor_xml_element_is_empty:
+ * @xml_element: XML Element
+ * 
+ * Check if an XML Element is empty.
+ * 
+ * Return value: non-0 if the element is empty.
+ **/
 int
 raptor_xml_element_is_empty(raptor_xml_element* xml_element)
 {
@@ -334,6 +342,8 @@ raptor_sax2_parse_start(raptor_sax2* sax2, raptor_uri *base_uri)
   sax2->root_element=NULL;
   sax2->current_element=NULL;
 
+  if(sax2->base_uri)
+    raptor_free_uri(sax2->base_uri);
   if(base_uri)
     sax2->base_uri=raptor_uri_copy(base_uri);
   else
@@ -387,6 +397,8 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
   
 #ifdef RAPTOR_XML_LIBXML
   if(!xc) {
+    int libxml_options = 0;
+
     if(!len) {
       /* no data given at all - emit a similar message to expat */
       raptor_sax2_update_document_locator(sax2, sax2->locator);
@@ -400,6 +412,10 @@ raptor_sax2_parse_chunk(raptor_sax2* sax2, const unsigned char *buffer,
                                  NULL);
     if(!xc)
       goto handle_error;
+
+    if(sax2->feature_no_net)
+      libxml_options |= XML_PARSE_NONET;
+    xmlCtxtUseOptions(xc, libxml_options);
     
     xc->userData = sax2; /* user data */
     xc->vctxt.userData = sax2; /* user data */
@@ -547,6 +563,10 @@ raptor_sax2_set_feature(raptor_sax2 *sax2, raptor_feature feature, int value)
       sax2->feature_normalize_language=value;
       break;
 
+    case RAPTOR_FEATURE_NO_NET:
+      sax2->feature_no_net=value;
+      break;
+
     case RAPTOR_FEATURE_SCANNING:
     case RAPTOR_FEATURE_ASSUME_IS_RDF:
     case RAPTOR_FEATURE_ALLOW_NON_NS_ATTRIBUTES:
