@@ -2,11 +2,8 @@
  *
  * raptor_serialize_rdfxml.c - RDF/XML serializer
  *
- * $Id: raptor_serialize_rdfxml.c,v 1.3 2005/11/26 06:39:57 dajobe Exp $
- *
- * Copyright (C) 2004-2005, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2004-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2004-2005, University of Bristol, UK http://www.bristol.ac.uk/
  *
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -70,6 +67,11 @@ typedef struct {
 
   /* URI of rdf:XMLLiteral */
   raptor_uri* rdf_xml_literal_uri;
+
+  /* non zero if rdf:RDF has been written (and thus no new namespaces
+   * can be declared).
+   */
+  int written_header;
 } raptor_rdfxml_serializer_context;
 
 
@@ -94,6 +96,8 @@ raptor_rdfxml_serialize_init(raptor_serializer* serializer, const char *name)
                                            0);
 
   context->namespaces=raptor_new_sequence(NULL, NULL);
+  /* Note: item 0 in the list is rdf:RDF's namespace */
+  raptor_sequence_push(context->namespaces, context->rdf_nspace);
 
   context->rdf_xml_literal_uri=raptor_new_uri(raptor_xml_literal_datatype_uri_string);
   
@@ -122,7 +126,8 @@ raptor_rdfxml_serialize_terminate(raptor_serializer* serializer)
   if(context->namespaces) {
     int i;
     
-    for(i=0; i< raptor_sequence_size(context->namespaces); i++) {
+    /* Note: item 0 in the list is rdf:RDF's namespace and freed above */
+    for(i=1; i< raptor_sequence_size(context->namespaces); i++) {
       raptor_namespace* ns=(raptor_namespace*)raptor_sequence_get_at(context->namespaces, i);
       if(ns)
         raptor_free_namespace(ns);
@@ -135,6 +140,47 @@ raptor_rdfxml_serialize_terminate(raptor_serializer* serializer)
 }
   
 
+#define RDFXML_NAMESPACE_DEPTH 0
+
+/* add a namespace */
+static int
+raptor_rdfxml_serialize_declare_namespace_from_namespace(raptor_serializer* serializer, 
+                                                         raptor_namespace *nspace)
+{
+  raptor_rdfxml_serializer_context* context=(raptor_rdfxml_serializer_context*)serializer->context;
+  int i;
+  
+  if(context->written_header)
+    return 1;
+  
+  for(i=0; i< raptor_sequence_size(context->namespaces); i++) {
+    raptor_namespace* ns;
+    ns=(raptor_namespace*)raptor_sequence_get_at(context->namespaces, i);
+
+    /* If prefix is already declared, ignore it */
+    if(!ns->prefix && !nspace->prefix)
+      return 1;
+    
+    if(ns->prefix && nspace->prefix && 
+       !strcmp((const char*)ns->prefix, (const char*)nspace->prefix))
+      return 1;
+
+    if(ns->uri && nspace->uri &&
+       raptor_uri_equals(ns->uri, nspace->uri))
+      return 1;
+  }
+
+  nspace=raptor_new_namespace_from_uri(context->nstack,
+                                       nspace->prefix, nspace->uri,
+                                       RDFXML_NAMESPACE_DEPTH);
+  if(!nspace)
+    return 1;
+  
+  raptor_sequence_push(context->namespaces, nspace);
+  return 0;
+}
+
+
 /* add a namespace */
 static int
 raptor_rdfxml_serialize_declare_namespace(raptor_serializer* serializer, 
@@ -143,13 +189,16 @@ raptor_rdfxml_serialize_declare_namespace(raptor_serializer* serializer,
 {
   raptor_rdfxml_serializer_context* context=(raptor_rdfxml_serializer_context*)serializer->context;
   raptor_namespace *ns;
+  int rc;
   
-  ns=raptor_new_namespace_from_uri(context->nstack, prefix, uri, 0);
-  if(!ns)
-    return 1;
+  ns=raptor_new_namespace_from_uri(context->nstack, prefix, uri, 
+                                   RDFXML_NAMESPACE_DEPTH);
   
-  raptor_sequence_push(context->namespaces, ns);
-  return 0;
+  rc=raptor_rdfxml_serialize_declare_namespace_from_namespace(serializer, 
+                                                               ns);
+  raptor_free_namespace(ns);
+  
+  return rc;
 }
 
 
@@ -159,12 +208,8 @@ raptor_rdfxml_serialize_start(raptor_serializer* serializer)
 {
   raptor_rdfxml_serializer_context* context=(raptor_rdfxml_serializer_context*)serializer->context;
   raptor_xml_writer* xml_writer;
-  raptor_uri *base_uri=serializer->base_uri;
-  int i;
   raptor_uri_handler *uri_handler;
   void *uri_context;
-  raptor_xml_element *element;
-  raptor_qname *qname;
 
   raptor_uri_get_handler(&uri_handler, &uri_context);
 
@@ -174,27 +219,49 @@ raptor_rdfxml_serialize_start(raptor_serializer* serializer)
                                    raptor_serializer_simple_error,
                                    serializer,
                                    1);
-  context->xml_writer=xml_writer;
-
   raptor_xml_writer_set_feature(xml_writer, RAPTOR_FEATURE_WRITER_XML_VERSION,
                                 serializer->xml_version);
   
+  context->xml_writer=xml_writer;
+  context->written_header=0;
+
+  return 0;
+}
+
+
+static void
+raptor_rdfxml_ensure_writen_header(raptor_serializer* serializer,
+                                   raptor_rdfxml_serializer_context* context) 
+{
+  raptor_xml_writer* xml_writer;
+  raptor_qname *qname;
+  raptor_uri *base_uri;
+  int i;
+
+  if(context->written_header)
+    return;
+  
+  xml_writer=context->xml_writer;
+
   qname=raptor_new_qname_from_namespace_local_name(context->rdf_nspace,
                                                    (const unsigned char*)"RDF",  NULL);
+  base_uri=serializer->base_uri;
   if(base_uri)
     base_uri=raptor_uri_copy(base_uri);
-  element=raptor_new_xml_element(qname, NULL, base_uri);
-  context->rdf_RDF_element=element;
-  for(i=0; i< raptor_sequence_size(context->namespaces); i++) {
+  context->rdf_RDF_element=raptor_new_xml_element(qname, NULL, base_uri);
+
+  /* NOTE: Starts it item 1 as item 0 is the element's namespace (rdf) 
+   * and does not need to be declared
+   */
+  for(i=1; i< raptor_sequence_size(context->namespaces); i++) {
     raptor_namespace* ns=(raptor_namespace*)raptor_sequence_get_at(context->namespaces, i);
     raptor_xml_element_declare_namespace(context->rdf_RDF_element, ns);
   }
   
-
   raptor_xml_writer_start_element(xml_writer, context->rdf_RDF_element);
   raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
 
-  return 0;
+  context->written_header=1;
 }
 
 
@@ -224,7 +291,10 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
   int attrs_count=0;
   raptor_uri* base_uri=NULL;
   raptor_identifier_type object_type;
+  int allocated=1;
   
+  raptor_rdfxml_ensure_writen_header(serializer, context);
+
   if(statement->predicate_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
     predicate_ns=context->rdf_nspace;
     sprintf((char*)ordinal_name, "_%d", *((int*)statement->predicate));
@@ -288,6 +358,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
 
     case RAPTOR_IDENTIFIER_TYPE_RESOURCE:
     case RAPTOR_IDENTIFIER_TYPE_ORDINAL:
+      allocated=1;
       if(statement->subject_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
         subject_uri_string=(unsigned char*)RAPTOR_MALLOC(cstring, raptor_rdf_namespace_uri_len+13);
         sprintf((char*)subject_uri_string, "%s_%d", 
@@ -296,12 +367,15 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
         if(serializer->feature_relative_uris)
           subject_uri_string=raptor_uri_to_relative_uri_string(serializer->base_uri,
                                                               (raptor_uri*)statement->subject);
-        else
+        else {
           subject_uri_string=raptor_uri_as_string((raptor_uri*)statement->subject);
+          allocated=0;
+        }
       }
       
       attrs[attrs_count++]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"about",  subject_uri_string);
-      RAPTOR_FREE(cstring, subject_uri_string);
+      if(allocated)
+        RAPTOR_FREE(cstring, subject_uri_string);
       
       break;
       
@@ -399,7 +473,7 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
 
     case RAPTOR_IDENTIFIER_TYPE_RESOURCE:
     case RAPTOR_IDENTIFIER_TYPE_ORDINAL:
-
+      allocated=1;
       if(object_type == RAPTOR_IDENTIFIER_TYPE_ORDINAL) {
         object_uri_string=(unsigned char*)RAPTOR_MALLOC(cstring, raptor_rdf_namespace_uri_len+13);
         sprintf((char*)object_uri_string, "%s_%d",
@@ -409,12 +483,15 @@ raptor_rdfxml_serialize_statement(raptor_serializer* serializer,
         if(serializer->feature_relative_uris)
           object_uri_string=raptor_uri_to_relative_uri_string(serializer->base_uri,
                                                               (raptor_uri*)statement->object);
-        else
-          object_uri_string=raptor_uri_as_string((raptor_uri*)statement->object);
+        else {
+          object_uri_string=raptor_uri_to_string((raptor_uri*)statement->object);
+          allocated=0;
+        }
       }
       
       attrs[attrs_count++]=raptor_new_qname_from_namespace_local_name(context->rdf_nspace, (const unsigned char*)"resource", object_uri_string);
-      RAPTOR_FREE(cstring, object_uri_string);
+      if(allocated)
+        RAPTOR_FREE(cstring, object_uri_string);
 
       raptor_xml_element_set_attributes(predicate_element, attrs, 1);
 
@@ -454,6 +531,8 @@ raptor_rdfxml_serialize_end(raptor_serializer* serializer)
   raptor_rdfxml_serializer_context* context=(raptor_rdfxml_serializer_context*)serializer->context;
   raptor_xml_writer* xml_writer=context->xml_writer;
 
+  raptor_rdfxml_ensure_writen_header(serializer, context);
+
   raptor_xml_writer_end_element(xml_writer, context->rdf_RDF_element);
   raptor_xml_writer_raw_counted(xml_writer, (const unsigned char*)"\n", 1);
 
@@ -479,6 +558,7 @@ raptor_rdfxml_serializer_register_factory(raptor_serializer_factory *factory)
   factory->init                = raptor_rdfxml_serialize_init;
   factory->terminate           = raptor_rdfxml_serialize_terminate;
   factory->declare_namespace   = raptor_rdfxml_serialize_declare_namespace;
+  factory->declare_namespace_from_namespace   = raptor_rdfxml_serialize_declare_namespace_from_namespace;
   factory->serialize_start     = raptor_rdfxml_serialize_start;
   factory->serialize_statement = raptor_rdfxml_serialize_statement;
   factory->serialize_end       = raptor_rdfxml_serialize_end;
