@@ -2,11 +2,8 @@
  *
  * raptor_uri.c - Raptor URI resolving implementation
  *
- * $Id: raptor_uri.c,v 1.89 2005/09/21 22:37:08 cmdjb Exp $
- *
- * Copyright (C) 2002-2005, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2002-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2002-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -73,9 +70,9 @@ static void *raptor_uri_current_uri_context;
  * @handler URI implementation.
  **/
 void
-raptor_uri_set_handler(raptor_uri_handler *handler, void *context) 
+raptor_uri_set_handler(const raptor_uri_handler *handler, void *context) 
 {
-  raptor_uri_current_uri_handler=handler;
+  raptor_uri_current_uri_handler=(raptor_uri_handler *)handler;
   raptor_uri_current_uri_context=context;
 }
 
@@ -103,7 +100,7 @@ raptor_default_new_uri(void *context, const unsigned char *uri_string)
   size_t len;
   
   /* If uri_string is "file:path-to-file", turn it into a correct file:URI */
-  if(raptor_uri_is_file_uri(uri_string)) {
+  if(raptor_uri_uri_string_is_file_uri(uri_string)) {
     unsigned char *fragment=NULL;
     char *filename;
     raptor_uri* uri=NULL;
@@ -116,7 +113,7 @@ raptor_default_new_uri(void *context, const unsigned char *uri_string)
         unsigned char *new_fragment;
         raptor_uri* new_uri;
 
-        new_fragment=(unsigned char*)RAPTOR_MALLOC(cstring, strlen((const char*)fragment)+2);
+        new_fragment=(unsigned char*)RAPTOR_MALLOC(cstring, strlen((const char*)fragment) + 1 + sizeof(char*));
         *new_fragment='#';
         strcpy((char*)new_fragment+1, (const char*)fragment);
         new_uri=raptor_new_uri_relative_to_base(uri, new_fragment);
@@ -134,7 +131,7 @@ raptor_default_new_uri(void *context, const unsigned char *uri_string)
   }
 
   len=strlen((const char*)uri_string);
-  p=(unsigned char*)RAPTOR_MALLOC(raptor_uri, len+1);
+  p=(unsigned char*)RAPTOR_MALLOC(raptor_uri, len + sizeof(char*));
   if(!p)
     return NULL;
   strcpy((char*)p, (const char*)uri_string);
@@ -153,6 +150,9 @@ raptor_default_new_uri(void *context, const unsigned char *uri_string)
 raptor_uri*
 raptor_new_uri(const unsigned char *uri_string) 
 {
+  if(!uri_string || !*uri_string)
+    return NULL;
+  
   return (*raptor_uri_current_uri_handler->new_uri)(raptor_uri_current_uri_context, uri_string);
 }
 
@@ -164,7 +164,7 @@ raptor_default_new_uri_from_uri_local_name(void *context,
 {
   int uri_length=strlen((char*)uri);
   unsigned char *p=(unsigned char*)RAPTOR_MALLOC(cstring, 
-                                                 uri_length + strlen((const char*)local_name) + 1);
+                                                 uri_length + strlen((const char*)local_name) + sizeof(char*));
   if(!p)
     return NULL;
   
@@ -191,6 +191,9 @@ raptor_default_new_uri_from_uri_local_name(void *context,
 raptor_uri*
 raptor_new_uri_from_uri_local_name(raptor_uri *uri, const unsigned char *local_name)
 {
+  if(!uri || !local_name)
+    return NULL;
+  
   return (*raptor_uri_current_uri_handler->new_uri_from_uri_local_name)(raptor_uri_current_uri_context, uri, local_name);
 }
 
@@ -201,7 +204,7 @@ raptor_default_new_uri_relative_to_base(void *context,
                                         const unsigned char *uri_string) 
 {
   raptor_uri* new_uri;
-  size_t new_uri_len=strlen((const char*)base_uri)+strlen((const char*)uri_string)+1;
+  size_t new_uri_len=strlen((const char*)base_uri)+strlen((const char*)uri_string) + sizeof(char*);
 
   /* +2 is for \0 plus an extra 1 for adding any missing URI path '/' */
   new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, new_uri_len+2);
@@ -233,6 +236,9 @@ raptor_uri*
 raptor_new_uri_relative_to_base(raptor_uri *base_uri, 
                                 const unsigned char *uri_string) 
 {
+  if(!base_uri || !uri_string)
+    return NULL;
+
   return (*raptor_uri_current_uri_handler->new_uri_relative_to_base)(raptor_uri_current_uri_context, base_uri, uri_string);
 }
 
@@ -256,12 +262,15 @@ raptor_new_uri_from_id(raptor_uri *base_uri, const unsigned char *id)
   unsigned char *local_name;
   int len;
 
+  if(!base_uri || !id)
+    return NULL;
+
 #if defined(RAPTOR_DEBUG) && RAPTOR_DEBUG > 1
   RAPTOR_DEBUG2("Using ID %s\n", id);
 #endif
 
   /* "#id\0" */
-  len=1+strlen((char*)id)+1;
+  len=1+strlen((char*)id) + sizeof(char*);
   local_name=(unsigned char*)RAPTOR_MALLOC(cstring, len);
   if(!local_name)
     return NULL;
@@ -281,7 +290,7 @@ raptor_default_new_uri_for_rdf_concept(void *context, const char *name)
   unsigned int base_uri_len=raptor_rdf_namespace_uri_len;
   unsigned int new_uri_len;
 
-  new_uri_len=base_uri_len+strlen(name)+1;
+  new_uri_len=base_uri_len+strlen(name) + sizeof(char*);
   new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, new_uri_len);
   if(!new_uri)
     return NULL;
@@ -305,6 +314,9 @@ raptor_default_new_uri_for_rdf_concept(void *context, const char *name)
 raptor_uri*
 raptor_new_uri_for_rdf_concept(const char *name) 
 {
+  if(!name)
+    return NULL;
+  
   return (*raptor_uri_current_uri_handler->new_uri_for_rdf_concept)(raptor_uri_current_uri_context, name);
 }
 
@@ -365,7 +377,7 @@ raptor_uri_equals(raptor_uri* uri1, raptor_uri* uri2)
 static raptor_uri*
 raptor_default_uri_copy(void *context, raptor_uri *uri)
 {
-  raptor_uri* new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, strlen((char*)uri)+1);
+  raptor_uri* new_uri=(raptor_uri*)RAPTOR_MALLOC(cstring, strlen((char*)uri) + sizeof(char*));
   if(!new_uri)
     return NULL;
   strcpy((char*)new_uri, (char*)uri);
@@ -385,6 +397,9 @@ raptor_default_uri_copy(void *context, raptor_uri *uri)
 raptor_uri*
 raptor_uri_copy(raptor_uri *uri) 
 {
+  if(!uri)
+    return NULL;
+  
   return (*raptor_uri_current_uri_handler->uri_copy)(raptor_uri_current_uri_context, uri);
 }
 
@@ -411,6 +426,9 @@ raptor_default_uri_as_string(void *context, raptor_uri *uri)
 unsigned char*
 raptor_uri_as_string(raptor_uri *uri) 
 {
+  if(!uri)
+    return NULL;
+  
   return (*raptor_uri_current_uri_handler->uri_as_string)(raptor_uri_current_uri_context, uri);
 }
 
@@ -442,6 +460,9 @@ raptor_default_uri_as_counted_string(void *context, raptor_uri *uri,
 unsigned char*
 raptor_uri_as_counted_string(raptor_uri *uri, size_t* len_p) 
 {
+  if(!uri)
+    return NULL;
+  
   return (*raptor_uri_current_uri_handler->uri_as_counted_string)(raptor_uri_current_uri_context, uri, len_p);
 }
 
@@ -469,7 +490,10 @@ raptor_uri_filename_to_uri_string(const char *filename)
   char path[PATH_MAX];
 #endif
   /*     "file://" ... \0 */
-  size_t len=7+1;
+  size_t len=7 + sizeof(char*);
+  
+  if(!filename)
+    return NULL;
   
 #ifdef WIN32
 /*
@@ -552,20 +576,6 @@ raptor_uri_filename_to_uri_string(const char *filename)
   }
   *to='\0';
   
-#ifdef RAPTOR_DEBUG
-  if(1) {
-    size_t actual_len=strlen((const char*)buffer)+1;
-    if(actual_len != len) {
-      if(actual_len > len)
-        RAPTOR_FATAL3("uri length %d is LONGER than malloced %d\n", 
-                      (int)actual_len, (int)len);
-      else
-        RAPTOR_DEBUG4("uri '%s' length %d is shorter than malloced %d\n", 
-                      buffer, (int)actual_len, (int)len);
-    }
-  }
-#endif
-
   return buffer;
 }
 
@@ -598,6 +608,9 @@ raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string,
   unsigned char *p;
 #endif
 
+  if(!uri_string || !*uri_string)
+    return NULL;
+  
   ud=raptor_new_uri_detail(uri_string);
   if(!ud)
     return NULL;
@@ -665,7 +678,7 @@ raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string,
     return NULL;
   }
     
-  filename=(char*)RAPTOR_MALLOC(cstring, len+1);
+  filename=(char*)RAPTOR_MALLOC(cstring, len + sizeof(char*));
   if(!filename) {
     raptor_free_uri_detail(ud);
     return NULL;
@@ -700,12 +713,12 @@ raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string,
 #endif
     if(c == '%') {
       if(*from && from[1]) {
-        unsigned char hexbuf[3];
-        unsigned char *endptr=NULL;
-        hexbuf[0]=*from;
-        hexbuf[1]=from[1];
+        char hexbuf[3];
+        char *endptr=NULL;
+        hexbuf[0]=(char)*from;
+        hexbuf[1]=(char)from[1];
         hexbuf[2]='\0';
-        c=(char)strtol((const char*)hexbuf, (char**)&endptr, 16);
+        c=(char)strtol((const char*)hexbuf, &endptr, 16);
         if(endptr == &hexbuf[2])
           *to++ = c;
       }
@@ -715,25 +728,10 @@ raptor_uri_uri_string_to_filename_fragment(const unsigned char *uri_string,
   }
   *to='\0';
 
-#ifdef RAPTOR_DEBUG
-  if(1) {
-    size_t actual_len=strlen(filename);
-    if(actual_len != len) {
-      if(actual_len > len) {
-        fprintf(stderr, "filename '%s'\n", filename);
-        RAPTOR_FATAL3("Filename length %d is LONGER than malloced %d\n", 
-                      (int)actual_len, (int)len);
-      } else
-        RAPTOR_DEBUG3("Filename length %d is shorter than malloced %d\n", 
-                      (int)actual_len, (int)len);
-    }
-  }
-#endif
-
   if(fragment_p) {
     if(ud->fragment) {
       len=ud->fragment_len;
-      *fragment_p=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+      *fragment_p=(unsigned char*)RAPTOR_MALLOC(cstring, len + sizeof(char*));
       strncpy((char*)*fragment_p, (const char*)ud->fragment, len+1);
     } else
       *fragment_p=NULL;
@@ -767,12 +765,31 @@ raptor_uri_uri_string_to_filename(const unsigned char *uri_string)
  * raptor_uri_is_file_uri:
  * @uri_string: The URI string to check
  *
- * Check if a URI string is a a file: URI.
- * 
+ * @Deprecated: use raptor_uri_uri_string_is_file_uri
+ *
+ * Check if a URI string is a file: URI.
+ *
  * Return value: Non zero if URI string is a file: URI
  **/
 int
 raptor_uri_is_file_uri(const unsigned char* uri_string) {
+  return raptor_uri_uri_string_is_file_uri(uri_string);
+}
+
+
+/**
+ * raptor_uri_uri_string_is_file_uri:
+ * @uri_string: The URI string to check
+ *
+ * Check if a URI string is a file: URI.
+ * 
+ * Return value: Non zero if URI string is a file: URI
+ **/
+int
+raptor_uri_uri_string_is_file_uri(const unsigned char* uri_string) {
+  if(!uri_string || !*uri_string)
+    return 1;
+
   return raptor_strncasecmp((const char*)uri_string, "file:", 5)==0;
 }
 
@@ -791,11 +808,16 @@ raptor_uri_is_file_uri(const unsigned char* uri_string) {
 raptor_uri*
 raptor_new_uri_for_xmlbase(raptor_uri* old_uri)
 {
-  unsigned char *uri_string=raptor_uri_as_string(old_uri);
+  unsigned char *uri_string;
   unsigned char *new_uri_string;
   raptor_uri* new_uri;
   raptor_uri_detail *ud;
   
+  if(!old_uri)
+    return NULL;
+
+  uri_string=raptor_uri_as_string(old_uri);
+
   ud=raptor_new_uri_detail(uri_string);
   if(!ud)
     return NULL;
@@ -833,11 +855,15 @@ raptor_new_uri_for_xmlbase(raptor_uri* old_uri)
 raptor_uri*
 raptor_new_uri_for_retrieval(raptor_uri* old_uri)
 {
-  unsigned char *uri_string=raptor_uri_as_string(old_uri);
+  unsigned char *uri_string;
   unsigned char *new_uri_string;
   raptor_uri* new_uri;
   raptor_uri_detail *ud;
   
+  if(!old_uri)
+    return NULL;
+
+  uri_string=raptor_uri_as_string(old_uri);
 
   ud=raptor_new_uri_detail(uri_string);
   if(!ud)
@@ -861,26 +887,7 @@ raptor_new_uri_for_retrieval(raptor_uri* old_uri)
 }
 
 
-void
-raptor_uri_init_default_handler(raptor_uri_handler *handler) 
-{
-  if(handler->initialised)
-    return;
-  
-  handler->new_uri=raptor_default_new_uri;
-  handler->new_uri_from_uri_local_name=raptor_default_new_uri_from_uri_local_name;
-  handler->new_uri_relative_to_base=raptor_default_new_uri_relative_to_base;
-  handler->new_uri_for_rdf_concept=raptor_default_new_uri_for_rdf_concept;
-  handler->free_uri=raptor_default_free_uri;
-  handler->uri_equals=raptor_default_uri_equals;
-  handler->uri_copy=raptor_default_uri_copy;
-  handler->uri_as_string=raptor_default_uri_as_string;
-  handler->uri_as_counted_string=raptor_default_uri_as_counted_string;
-  
-  handler->initialised=1;
-}
-
-static raptor_uri_handler raptor_uri_default_handler = {
+static const raptor_uri_handler raptor_uri_default_handler = {
   raptor_default_new_uri,
   raptor_default_new_uri_from_uri_local_name,
   raptor_default_new_uri_relative_to_base,
@@ -897,7 +904,6 @@ static raptor_uri_handler raptor_uri_default_handler = {
 void
 raptor_uri_init(void)
 {
-  raptor_uri_init_default_handler(&raptor_uri_default_handler);
   raptor_uri_set_handler(&raptor_uri_default_handler, NULL);
 }
 
@@ -995,7 +1001,7 @@ raptor_uri_path_make_relative_path(const unsigned char *from_path, size_t from_p
   
   /* Create the final relative path */
   final_len = up_dirs*3 + to_dir_len + suffix_len; /* 3 for each "../" */
-  final_path=(unsigned char*)RAPTOR_MALLOC(cstring, final_len + 1);
+  final_path=(unsigned char*)RAPTOR_MALLOC(cstring, final_len + sizeof(char*));
   *final_path=0;
   
   /* First, add the necessary "../" parts */
@@ -1041,7 +1047,7 @@ raptor_uri_path_make_relative_path(const unsigned char *from_path, size_t from_p
 
 /**
  * raptor_uri_to_relative_counted_uri_string:
- * @base_uri: The base absolute URI to resolve against
+ * @base_uri: The base absolute URI to resolve against (or NULL)
  * @reference_uri: The reference absolute URI to use
  * @length_p: Location to store the length of the relative URI string or NULL
  *
@@ -1060,12 +1066,12 @@ raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri,
   size_t base_len, reference_len, reference_file_len, prefix_len, suffix_len;
   unsigned char *result=NULL;
 
-  if(length_p)
-    *length_p=0;
-
   if(!reference_uri)
     return NULL;
     
+  if(length_p)
+    *length_p=0;
+
   reference_str=raptor_uri_as_counted_string(reference_uri, &reference_len);
   reference_detail=raptor_new_uri_detail(reference_str);
   
@@ -1133,7 +1139,7 @@ raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri,
       suffix_len++; /* add one char for the '#' */
     
     /* Assemble the suffix */
-    suffix=(unsigned char*)RAPTOR_MALLOC(cstring, suffix_len + 1);
+    suffix=(unsigned char*)RAPTOR_MALLOC(cstring, suffix_len + sizeof(char*));
     cur_ptr=suffix;
     if(reference_file) {
       memcpy(suffix, reference_file, reference_file_len);
@@ -1169,7 +1175,7 @@ raptor_uri_to_relative_counted_uri_string(raptor_uri *base_uri,
   /* If result is NULL at this point, it means that we were unable to find a
      relative URI, so we'll return a full absolute URI instead. */
   if(!result) {
-    result=(unsigned char*)RAPTOR_MALLOC(cstring, reference_len+1);
+    result=(unsigned char*)RAPTOR_MALLOC(cstring, reference_len + sizeof(char*));
     if(reference_len)
       memcpy(result, reference_str, reference_len);
     result[reference_len] = 0;
@@ -1212,9 +1218,12 @@ raptor_uri_to_relative_uri_string(raptor_uri *base_uri,
  **/
 void
 raptor_uri_print(const raptor_uri* uri, FILE *stream) {
-  size_t len;
-  unsigned char *string=raptor_uri_as_counted_string((raptor_uri*)uri, &len);
-  (void)fwrite(string, len, 1, stream);
+  if(uri) {
+    size_t len;
+    unsigned char *string=raptor_uri_as_counted_string((raptor_uri*)uri, &len);
+    (void)fwrite(string, len, 1, stream);
+  } else 
+    (void)fwrite("(NULL URI)", 10, 1, stream);
 }
 
 
@@ -1239,11 +1248,14 @@ raptor_uri_to_counted_string(raptor_uri *uri, size_t *len_p)
   unsigned char *string;
   unsigned char *new_string;
 
+  if(!uri)
+    return NULL;
+  
   string=raptor_uri_as_counted_string(uri, &len);
   if(!string)
     return NULL;
   
-  new_string=(unsigned char*)RAPTOR_MALLOC(cstring, len+1);
+  new_string=(unsigned char*)RAPTOR_MALLOC(cstring, len + sizeof(char*));
   if(!new_string)
     return NULL;
   
@@ -1273,6 +1285,27 @@ raptor_uri_to_string(raptor_uri *uri)
 }
 
 
+/**
+ * raptor_new_uri_from_rdf_ordinal:
+ * @ordinal: integer rdf:_n
+ * 
+ * Internal - convert an integer rdf:_n ordinal to the resource URI
+ * 
+ * Return value: new URI object or NULL on failure
+ **/
+raptor_uri*
+raptor_new_uri_from_rdf_ordinal(int ordinal)
+{
+  /* 55 = strlen(rdf namespace URI) + _ + 10-digit number + \0 */
+  unsigned char uri_string[55];
+  strncpy((char*)uri_string, (const char*)raptor_rdf_namespace_uri, 
+          raptor_rdf_namespace_uri_len);
+  sprintf((char*)uri_string+raptor_rdf_namespace_uri_len, "_%d",
+          ordinal);
+  return raptor_new_uri(uri_string);
+}
+
+
 #ifdef STANDALONE
 
 #include <stdio.h>
