@@ -2,11 +2,8 @@
  *
  * rdfdump.c - Raptor RDF Parser example code 
  *
- * $Id: rapper.c,v 1.113 2005/09/06 21:06:17 dajobe Exp $
- *
- * Copyright (C) 2000-2005, David Beckett http://purl.org/net/dajobe/
- * Institute for Learning and Research Technology http://www.ilrt.bristol.ac.uk/
- * University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -63,8 +60,9 @@ extern int optind;
 extern char *optarg;
 #endif
 
-static void print_statements(void *user_data, const raptor_statement *statement);
+static void print_triples(void *user_data, const raptor_statement *statement);
 static void print_namespaces(void* user_data, raptor_namespace *nspace);
+static void relay_namespaces(void* user_data, raptor_namespace *nspace);
 int main(int argc, char *argv[]);
 
 
@@ -78,7 +76,7 @@ static int quiet=0;
 /* just count, no printing */
 static int count=0;
 
-static int statement_count=0;
+static int triple_count=0;
 
 static raptor_serializer* serializer=NULL;
 
@@ -89,10 +87,10 @@ static int reported_guess=0;
 static int report_namespace=0;
 
 static
-void print_statements(void *user_data, const raptor_statement *statement) 
+void print_triples(void *user_data, const raptor_statement *triple) 
 {
   raptor_parser* rdf_parser=(raptor_parser*)user_data;
-  statement_count++;
+  triple_count++;
 
   if(guess && !quiet && !reported_guess) {
      fprintf(stdout, "%s: Guessed parser name '%s'\n", program,
@@ -106,14 +104,14 @@ void print_statements(void *user_data, const raptor_statement *statement)
 
   /* replace newlines with spaces if object is a literal string */
   if(replace_newlines && 
-     statement->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL) {
+     triple->object_type == RAPTOR_IDENTIFIER_TYPE_LITERAL) {
     char *s;
-    for(s=(char*)statement->object; *s; s++)
+    for(s=(char*)triple->object; *s; s++)
       if(*s == '\n')
         *s=' ';
   }
 
-  raptor_serialize_statement(serializer, statement);
+  raptor_serialize_statement(serializer, triple);
   return;
 }
 
@@ -135,6 +133,18 @@ print_namespaces(void* user_data, raptor_namespace *nspace)
 }
 
 
+static void
+relay_namespaces(void* user_data, raptor_namespace *nspace)
+{
+  raptor_serializer* rdf_serializer=(raptor_serializer*)user_data;
+
+  if(report_namespace)
+    print_namespaces(user_data, nspace);
+
+  raptor_serialize_set_namespace_from_namespace(rdf_serializer, nspace);
+}
+
+
 
 #ifdef HAVE_GETOPT_LONG
 #define HELP_TEXT(short, long, description) "  -" short ", --" long "  " description
@@ -183,7 +193,7 @@ static int warning_count=0;
 static int ignore_warnings=0;
 static int ignore_errors=0;
 
-static const char *title_format_string="Raptor RDF parser utility %s\n";
+static const char *title_format_string="Raptor RDF syntax parsing and serializing utility %s\n";
 
 
 static void
@@ -300,7 +310,7 @@ main(int argc, char *argv[])
             int i;
             
             fprintf(stderr, "%s: Valid parser features are:\n", program);
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < raptor_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
               if(!raptor_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
@@ -310,7 +320,7 @@ main(int argc, char *argv[])
               }
             }
             fprintf(stderr, "%s: Valid serializer features are:\n", program);
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < raptor_get_feature_count(); i++) {
               const char *feature_name;
               const char *feature_label;
               if(!raptor_serializer_features_enumerate((raptor_feature)i, &feature_name, NULL, &feature_label)) {
@@ -321,6 +331,9 @@ main(int argc, char *argv[])
             }
             fputs("Features are set with `" HELP_ARG(f, feature) " FEATURE=VALUE or `-f FEATURE'\nand take a decimal integer VALUE except where noted, defaulting to 1 if omitted.\n", stderr);
             fputs("\nA feature of the form xmlns:PREFIX=\"URI\" can be used to declare output\nnamespace prefixes and names for serializing using an XML-style syntax\nEither or both of PREFIX or URI can be omitted such as -f xmlns=\"URI\"\nThis form can be repeated for multiple declarations.\n", stderr);
+
+            raptor_finish();
+
             exit(0);
           } else if(!strncmp(optarg, "xmlns", 5)) {
             struct namespace_decl *nd;
@@ -342,7 +355,7 @@ main(int argc, char *argv[])
             int i;
             size_t arg_len=strlen(optarg);
             
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < raptor_get_feature_count(); i++) {
               const char *feature_name;
               size_t len;
               
@@ -366,7 +379,7 @@ main(int argc, char *argv[])
               }
             }
             
-            for(i=0; i <= RAPTOR_FEATURE_LAST; i++) {
+            for(i=0; i < raptor_get_feature_count(); i++) {
               const char *feature_name;
               size_t len;
               
@@ -498,6 +511,9 @@ main(int argc, char *argv[])
       case 'v':
         fputs(raptor_version_string, stdout);
         fputc('\n', stdout);
+
+        raptor_finish();
+
         exit(0);
 
 #ifdef SHOW_NAMESPACES_FLAG
@@ -523,6 +539,8 @@ main(int argc, char *argv[])
     }
     fprintf(stderr, "Try `%s " HELP_ARG(h, help) "' for more information.\n",
                     program);
+    raptor_finish();
+
     exit(1);
   }
 
@@ -533,8 +551,7 @@ main(int argc, char *argv[])
     printf(title_format_string, raptor_version_string);
     puts(raptor_short_copyright_string);
     puts("Parse RDF content at the source URI into RDF triples.");
-    puts("\nMain options:");
-    puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
+    puts("\nSyntax options:");
     puts(HELP_TEXT("i FORMAT", "input FORMAT ", "Set the input format to one of:"));
     for(i=0; 1; i++) {
       const char *help_name;
@@ -559,12 +576,13 @@ main(int argc, char *argv[])
       else
         putchar('\n');
     }
-    puts(HELP_TEXT("m MODE", "mode MODE  ", "Set parser mode - 'lax' (default) or 'strict'"));
-    puts("\nAdditional options:");
+    puts("\nGeneral options:");
     puts(HELP_TEXT("c", "count           ", "Count triples only - do not print them."));
     puts(HELP_TEXT("e", "ignore-errors   ", "Ignore error messages"));
     puts(HELP_TEXT("f FEATURE(=VALUE)", "feature FEATURE(=VALUE)", HELP_PAD "Set parser or serializer features" HELP_PAD "Use `-f help' for a list of valid features"));
     puts(HELP_TEXT("g", "guess           ", "Guess the input syntax (same as -i guess)"));
+    puts(HELP_TEXT("h", "help            ", "Print this help, then exit"));
+    puts(HELP_TEXT("m MODE", "mode MODE  ", "Set parser mode - 'lax' (default) or 'strict'"));
     puts(HELP_TEXT("q", "quiet           ", "No extra information messages"));
     puts(HELP_TEXT("r", "replace-newlines", "Replace newlines with spaces in literals"));
     puts(HELP_TEXT("s", "scan            ", "Scan for <rdf:RDF> element in source"));
@@ -673,9 +691,10 @@ main(int argc, char *argv[])
     }
   }
   
-  raptor_set_statement_handler(rdf_parser, rdf_parser, print_statements);
+  raptor_set_statement_handler(rdf_parser, rdf_parser, print_triples);
 
-  raptor_set_namespace_handler(rdf_parser, rdf_parser, print_namespaces);
+  if(report_namespace)
+    raptor_set_namespace_handler(rdf_parser, rdf_parser, print_namespaces);
 
 
   if(serializer_syntax_name) {    
@@ -713,6 +732,8 @@ main(int argc, char *argv[])
 
     raptor_serialize_start_to_file_handle(serializer, base_uri, stdout);
 
+    if(!report_namespace)
+      raptor_set_namespace_handler(rdf_parser, serializer, relay_namespaces);
   }
   
 
@@ -741,8 +762,8 @@ main(int argc, char *argv[])
   
 
   if(!quiet)
-    fprintf(stdout, "%s: Parsing returned %d statements\n", program,
-            statement_count);
+    fprintf(stdout, "%s: Parsing returned %d triple%s\n", program,
+            triple_count, (triple_count == 1 ? "" : "s"));
 
   raptor_free_uri(base_uri);
   if(uri)
